package catnet.agent;

import java.util.*;
import java.lang.*;
import drcl.comp.Port;
import drcl.comp.Contract;
import drcl.inet.InetPacket;
import catnet.negotiation.Conversation;
import catnet.data.*;
import catnet.strategy.*;
import catnet.util.*;
import catnet.log.store;
import java.sql.*;
import catnet.util.TM_EVT;
import catnet.learning.stdea.Genotype;
import catnet.learning.stdea.Plumage;
import catnet.learning.ILearning;

/**
 *  Baseline Super_Class
 *
 *@author     eymann
 *@created    5. August 2002
 */

public abstract class Agent_Source extends drcl.inet.application.SUDPApplication {
    
    protected boolean ableToStartDemand = false;
    
    /**
     *  Description of the Field
     */
    final String CLASS_NAME = "Agent_Source";
    
    /**
     *  Description of the Field
     */
    
    //Current state of execution
    protected int executionState = 0;
    
    //Is this experiment baseline (only set by script)
    private boolean baseline=false;
    
    //Initial price, set by script
    private int default_price;
    
    //Generate HashMap for negotiations
    public HashMap negotiations = new HashMap();
    
    /**
     *  Description of the Field
     */
    //Set strategy
    public IPricing strategy = new catnet.strategy.avalanche();
    
    //Already forwarded messages
    public TreeSet forwarded_Conversations = new TreeSet();
    
    /**
     *  Description of the Field
     */
    public Hp LOCALHP = new Hp(-1, -1);
    // This has to be obtained from javasim !!
    
    /** Holds the pending cfps received */
    private LinkedList pendingNegotiations;
    private int negotiating = -1;
    
    // COMMON DATA STRUCTURES *****************************************
    
    /**
     *  Description of the Field
     */
    
    //Budget
    public double money_balance = 0;
    /**
     *  Description of the Field
     */
    public final int MAX = 100;
    
    //What items are tradeable (only for mislead messages)
    public Vector tradeable = new Vector();
    
    //What Plumages (depends on itemID) could be accepted
    protected Vector accept_plumage = new Vector();
    
    //Vector of openCfPs
    public Vector openCfPs = new Vector();

    protected double periodicalServicesTimeout;
    
    // CONSTRUCTORS
    /**
     *  Constructor for the Agent_Source object
     */
    public Agent_Source() {
        super();
        
        if (constant.distributed_nonegotiate)
            strategy = new catnet.strategy.allaccept();
        
        if (!constant.learning) {
            strategy.setMyGenotype( new Genotype(0.75, 0.5, 0.15, 0.75, 0.30, 1.0));
        } else {
            //Set default genotype, could be changed later by tcl-script
            strategy.randomizeMyGenotype();
        };
        
        //Set learningOwner and strategyOwner
        strategy.getLearningModule().setLearningOwner(this);

        strategy.setStrategyOwner(this);
        negotiations.clear();
        pendingNegotiations = new LinkedList();
        negotiating = -1;
        
    }
    
    //unique id for the resource/client/servicecopy agent
    public int ownID;
    
    // INITIALIZATION RESOURCE
    
    /**
     *  Sets the hP attribute of the Agent_Source object
     *
     *@param  addr  The new hP value
     *@param  port  The new hP value
     */
    public void setHP(long addr, int port) {
        LOCALHP.h = addr;
        LOCALHP.p = port;
        ownID = (int) (addr);
        store.log(CLASS_NAME, constant.log_infos, "setHP", ownID, this.LOCALHP.p, "Initial Genotype set to: " + strategy.getMyGenotype());
    }
    
    public IPricing getStrategy() {
        return this.strategy;
    }
    
    // COMMUNICATION UP / DOWN
    
    /**
     *  Description of the Method
     *
     *@param  data_      Description of the Parameter
     *@param  downPort_  Description of the Parameter
     */
    
    public synchronized void dataArriveAtDownPort(Object data_, Port downPort_) {
        String METHOD_NAME="dataArriveAtDownPort()";
        pkt pkt_ = (pkt) getContent(data_);
        if (constant.debuglog) Debug.say("Agent " + ownID + ":" + this.LOCALHP.p + " received from "+ pkt_.msg.sender_hp.h + ":" + pkt_.msg.sender_hp.p + " ", pkt_.type, " do ^ " + pkt_.msg.getConversation_ID());
        
        //Broadcast yes or no?
        boolean broadcast = pkt_.msg.broadcast;
        // IMPLEMENTATION OF BROADCAST by msg.broadcast check
        if (broadcast == true) {
            //This must be a broadcast message
            if (isDebugEnabled()) { debug("Broadcast message : pkt_.msg.hop_counter="+pkt_.msg.hop_counter+"  constant.hop_limit="+constant.hop_limit); };
            if (pkt_.msg.hop_counter < constant.hop_limit) {

                //Increase hop_counter and forward the message to the adjacent resources
                //Done by the concrete classes
                //pkt_.msg.hop_counter++;
                 if (!forwarded_Conversations.contains(pkt_.msg.getConversation_ID())) {
                    //System.out.println("Agent "+ownID+" has forwarded_Conversations: " + forwarded_Conversations+ " and forwards "+ pkt_.msg.getConversation_ID()+ ", Hop-Counter: "+pkt_.msg.hop_counter);    
                     forwarded_Conversations.add(pkt_.msg.getConversation_ID());

                     if (pkt_.type == constant.MSG_Plumage) {
                        //Learning algorithm
//                        System.out.println(); 
                        interpret_Plumage(pkt_.msg);
                    };                    
                    
                     interpret_BroadcastMessage(pkt_.type, pkt_.msg);

                } else {
                    //System.out.println("Agent "+ownID+" has forwarded_Conversations: " + forwarded_Conversations+ " and does not forward "+ pkt_.msg.getConversation_ID());
                };
            };
        };
        
        if ((pkt_.msg.hop_counter <= constant.hop_limit) && (broadcast == false)) {
            
            switch (pkt_.type) {
                case constant.MSG_Cfp:
                    interpretCfp(pkt_.msg);
                    break;
                case constant.MSG_RequestService:
                    interpretRequestService(pkt_.msg);
                    break;
                case constant.MSG_Accept:
                    interpretAccept(pkt_.msg);
                    break;
                case constant.MSG_Proposal:
                    interpretProposal(pkt_.msg);
                    break;
                case constant.MSG_Reject:
                    interpretReject(pkt_.msg);
                    break;
                case constant.MSG_informResourceHost:
                    interpretinformResourceHost(pkt_.msg);
                    break;
                case constant.MSG_RequestBandwidth:
                    interpretProposal(pkt_.msg);
                    break;
                case constant.MSG_Confirm:
                    interpretConfirm(pkt_.msg);
                    break;
                case constant.MSG_MoneyTransfer:
                    interpretMoneyTransfer(pkt_.msg);
                    break;
                case constant.MSG_Plumage:
                    interpret_Plumage(pkt_.msg);
                    break;
                case constant.MSG_ProvideService:
                    interpretProvideService(pkt_.msg);
                    break;
                case constant.MSG_Status:
                    interpretStatus(pkt_.msg);
                    break;
                default:
//                    Debug.say("DEFAULT " + ownID + ":" + this.LOCALHP.p + " received from "+ pkt_.msg.sender_hp.h + ":" + pkt_.msg.sender_hp.p + " ", pkt_.type, " do ^ " + pkt_.msg.getConversation_ID());
            }
        } // end if
    }
    
    
    /**
     *  Description of the Method
     *
     *@param  msgType  Description of the Parameter
     *@param  msg      Description of the Parameter
     *@param  hp       Description of the Parameter
     */
    public void goMsg_Dest(int msgType, Msg msg, Hp hp) {
        /*
         *  similar to send_echo_request()
         */
        //Debug.say("Agent " + ownID, msgType, "go v " + msg.getConversation_ID());
        if (!msg.broadcast) {
//            if (msg.receiver_hp.p != 101) {
                if (constant.debuglog) Debug.say("Agent " + ownID + ":" + this.LOCALHP.p + " sends to "+ msg.receiver_hp.h + ":"+ msg.receiver_hp.p, msgType, " go ^ " + msg.getConversation_ID());
//            }
//            else {
//                if (constant.debuglog) Debug.say("Agent " + ownID + ":" + this.LOCALHP.p + " sends to "+ msg.receiver_hp.h + " ", msgType, " go ^ " + msg.getConversation_ID());
//            };
       
        } else {
            if (constant.debuglog) Debug.say("Agent " + ownID + " sends a broadcast ", msgType, " go ^ " + msg.getConversation_ID());
        };
        
        sendmsg(new pkt(msgType, msg), 100, hp.h, hp.p);
    }
    
    /**
     *  Gets the price attribute of the Agent_Source object
     *
     *@return    The price value
     */
    public int getPrice() {
        return default_price;
    }
    
    /**
     *  Sets the price attribute of the Agent_Source object
     *
     *@param  price  The new price value
     */
    
    public void setPrice(Integer itemID, double price_min, double price_max) {
        //default_price = price;
        strategy.setPrices(itemID, price_min, price_max);
    }
    
    public void setGenotype(double p_acquisitiveness, double del_change, double del_jump, double p_satisfaction,
			double w_memory, double p_reputation, double probability) {

        strategy.setMyGenotype(new Genotype(p_acquisitiveness, del_change, del_jump, p_satisfaction, w_memory, p_reputation));
    }
    
    public void setBaseline(boolean bool) {
        if (bool) {
            baseline=true;
        }
        else {
            baseline=false;
        };
    }
    
    /**
     *  Description of the Method
     *
     *@param  currentMessage  Description of the Parameter
     *@return                 Description of the Return Value
     */
    
    protected void interpretRequestService(Msg currentMessage) {};
    
    protected synchronized boolean interpretAccept(Msg currentMessage) {
        //For loggging we need the name of the method
        String METHOD_NAME = "interpretAccept";
        boolean done = true;
        this.executionState = constant.NEGOTIATING;
        Msg replyMsg = null;
        
        Integer factorID = new Integer(currentMessage.itemID);
        Integer conversation_ID = currentMessage.getConversation_ID();
        
        /* -> Baseline */
        if ((!negotiations.containsKey(conversation_ID)) && (constant.m_baseline == false)) {
            /* <- Baseline */
            //require a proposal or cfp
            goMsg_Dest(constant.MSG_Reject,
            currentMessage.createReply(ownID, constant.MSG_Reject, currentMessage.switchDocumentType(), -1),
            currentMessage.sender_hp);
            store.log(this.CLASS_NAME, constant.log_errors, METHOD_NAME, ownID,  " NO CONVERSATION=" + currentMessage);
            this.executionState = constant.IDLE;
            return false;
        }
        
        if (constant.m_baseline == false) {
            Conversation currentConversation = (Conversation) negotiations.get(conversation_ID);
            Msg myCurrentProposal = (Msg) currentConversation.getLastSentMessage();
            store.log(this.CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, "Agent " + ownID + ": MSG Accept received, Conversation "+ conversation_ID);
            
            postAcceptanceMethod(currentMessage);
            
            this.executionState = constant.IDLE;
            
            currentConversation.setLastSentMessage(null);
            currentConversation.setLastReceivedMessage(currentMessage);
            negotiations.put(conversation_ID, currentConversation);
            
            learn(currentMessage.price-strategy.getAveragePrice(factorID), factorID);
        }
        return true;
    }
    
    public abstract void learn(double gain, Integer factorID);
    
    /**
     *  Description of the Method
     */
    protected void postRejectanceMethod(Msg msg) {};
    
    protected void postAcceptanceMethod(Msg msg) {};
    
    /**
     *  Description of the Method
     *
     *@param  currentMessage  Description of the Parameter
     *@return                 Description of the Return Value
     */
    protected synchronized boolean interpretReject(Msg currentMessage) {
        //For loggging we need the name of the method
        String METHOD_NAME = "interpretReject";
        boolean done = true;
        this.executionState = constant.NEGOTIATING;
        Msg replyMsg = null;
        Integer factorID = new Integer(currentMessage.itemID);
        Integer conversation_ID = currentMessage.getConversation_ID();
        
        /*
         *  if (!negotiations.containsKey(conversation_ID)) {
         *  /require a proposal or cfp
         *  goMsg_Dest(constant.MSG_Reject,
         *  currentMessage.createReply(ownID, constant.MSG_Reject, currentMessage.switchDocumentType(), -1),
         *  currentMessage.sender_hp);
         *  log(this.CLASS_NAME, constant.log_errors, METHOD_NAME, " NO CONVERSATION=" + currentMessage);
         *  this.executionState = constant.IDLE;
         *  return false;
         *  }
         */
        if (constant.m_baseline) {
             postRejectanceMethod(currentMessage);
            return true;
        }
        
//        Conversation currentConversation = (Conversation) negotiations.get(conversation_ID);
        if (isDebugEnabled()) { debug("Agent "+ownID+" conv_ID = " + conversation_ID.intValue() + ": rejected"); };
//        Msg myCurrentProposal = (Msg) currentConversation.getLastSentMessage();
        //possible: delete conversation from hashmap to clear data structures
        //How to clear it?
        store.log(CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, "Agent "+ownID+" Proposal rejected");
        
        // INBOX-Stack
        setNegotiating(false);
        if (anyPendingNegotiation()) {
            interpretCfp(getNextPendingNegotiation());
        } else {
            idleComputing();
        }
            
        postRejectanceMethod(currentMessage);
        
        //Delete negotiation
        negotiations.remove(conversation_ID);
        
        
        this.executionState = constant.IDLE;
        
        return true;
    }
    
    protected void doCfp(Conversation currentConversation) {};
    
    /**
     *  Description of the Method
     *
     *@param  currentMessage  Description of the Parameter
     *@return                 Description of the Return Value
     */
    protected synchronized boolean interpretProposal(Msg currentMessage) {
        //For logging we need the name of the method
        String METHOD_NAME = "interpretProposal";
        boolean done = true;
        if (currentMessage.DocumentType == constant.BID) { if (isDebugEnabled()) { debug("Agent: "+ownID+ " Received Message is a BID"); };};
        if (currentMessage.DocumentType == constant.ASK) { if (isDebugEnabled()) { debug("Agent: "+ownID+" Received Message is an ASK"); };};
        
        this.executionState = constant.NEGOTIATING;
        Msg replyMsg = null;
        
        Integer factorID = new Integer(currentMessage.itemID);
        Integer conversation_ID = currentMessage.getConversation_ID();
        // Check if we have closed any parallel conversation for this Item
        if ((!openCfPs.contains(currentMessage.request_ID)) && (currentMessage.itemID >= 2)) {
            //Error, Reject
            if (isDebugEnabled()) { debug(currentMessage.itemID+" has already been served."); };
            store.log(CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, currentMessage.itemID+" has been served.");
            replyMsg = currentMessage.createReply(ownID, constant.MSG_Reject, currentMessage.switchDocumentType(), 0);
            goMsg_Dest(constant.MSG_Reject, replyMsg, currentMessage.sender_hp);
            return false;
        };
        if (!negotiations.containsKey(conversation_ID)) {
            //require a cfp
            //if (isDebugEnabled()) { debug(conversation_ID); };
            goMsg_Dest(constant.MSG_Reject,
            currentMessage.createReply(ownID, constant.MSG_Reject, currentMessage.switchDocumentType(), -1),
            currentMessage.sender_hp);
            store.log(CLASS_NAME, constant.log_errors, METHOD_NAME, ownID, " NO CONVERSATION =" + currentMessage);
            this.executionState = constant.IDLE;
            return false;
        }
        Conversation currentConversation = (Conversation) negotiations.get(conversation_ID);
        Msg myCurrentProposal = (Msg) currentConversation.getLastSentMessage();
        
        /**
         *  check whether opponents price is -1, means mislead cfp if
         *  (currentMessage.price == -1) { goMsg_Dest(constant.MSG_Cfp,
         *  currentMessage, currentMessage.receiver_hp); return true; }
         */
        
        //if offers match, clear replyMsg
        Integer Integer_itemID = new Integer(currentMessage.itemID);
        //store.log(this.CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, "Offers match");
        
        if (checkRestrictions(currentMessage.itemID)) {
            
            // calling the strategy
            double myNewPrice = strategy.bargain(Integer_itemID, currentMessage.switchDocumentType(), currentMessage.price, myCurrentProposal.price, conversation_ID);
            if (constant.debuglog) System.out.println("myNewPrice= " + myNewPrice);
            if (isDebugEnabled()) { debug("Agent " + ownID + " | "+"Conv: " + conversation_ID.intValue() + ": myNewPrice in interpretPROPOSAL=" + myNewPrice); };
            store.log(this.CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, "myNewPrice in interpretPROPOSAL=" + myNewPrice);
            
            switch (strategy.getRecommendedAction()) {
                case constant.MSG_Proposal:
                    replyMsg = currentMessage.createReply(ownID, constant.MSG_Proposal, currentMessage.switchDocumentType(), myNewPrice);
                    store.log(this.CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, "SEND NEW PROPOSAL -- MESSAGE=" + replyMsg);
                    this.executionState = constant.NEGOTIATING;
                    break;
                case constant.MSG_Reject:
                    replyMsg = currentMessage.createReply(ownID, constant.MSG_Reject, currentMessage.switchDocumentType(), myNewPrice);
                    store.log(this.CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, "SEND REJECTED -- MESSAGE=" + replyMsg);
                    this.executionState = constant.IDLE;
                    break;
                case constant.MSG_Accept:
                    replyMsg = currentMessage.createReply(ownID, constant.MSG_Accept, currentMessage.switchDocumentType(), myNewPrice);
                    store.log(this.CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, "Proposal ("+ myNewPrice +") ACCEPTED");
                    this.executionState = constant.NEGOTIATING;
                    //Learn
                    learn(currentMessage.price-strategy.getAveragePrice(factorID), factorID);
                    break;
            }
            //if (isDebugEnabled()) { debug("Agent " + ownID + "| Conv: " + conversation_ID + ": replyMsg = " + replyMsg + ", curMsg= " + currentMessage); };
            store.log(CLASS_NAME, constant.log_business, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, "propose "+myNewPrice+" to Agent "+currentMessage.sender_hp.h+": "+currentMessage.sender_hp.p);
            currentConversation.setLastSentMessage(replyMsg);
            currentConversation.setLastReceivedMessage(currentMessage);
            negotiations.put(conversation_ID, currentConversation);
            
            goMsg_Dest(replyMsg.verdict, replyMsg, currentMessage.sender_hp);
            
            // INBOX-Stack
            if (replyMsg.verdict == constant.MSG_Reject) {
                setNegotiating(false);
                if (anyPendingNegotiation()) {
                    interpretCfp(getNextPendingNegotiation());
                } else {
                    idleComputing();
                }
                postRejectanceMethod(currentMessage);
                negotiations.remove(conversation_ID);
            }
            else if (replyMsg.verdict == constant.MSG_Accept)
                postAcceptanceMethod(currentMessage);

            return true;
        } else {
            //check restricitions = false
            store.log(CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, "CheckRestrictions is false");
            if (isDebugEnabled()) { debug("CheckRestrictions is false"); };
            do_Reject(currentMessage);
            postRejectanceMethod(currentMessage);
            
            // INBOX-Stack
            setNegotiating(false);
            if (anyPendingNegotiation()) {
                interpretCfp(getNextPendingNegotiation());
            } else {
                idleComputing();
            }
            
            return false;
        }
    }
    
    protected void do_Reject(Msg currentMessage) {
        Msg msg=new Msg();
        msg.receiver_hp=currentMessage.sender_hp;
        msg.verdict=constant.MSG_Reject;
        msg.sender_hp=new Hp(LOCALHP);
        msg.itemID = currentMessage.itemID;
        msg.setConversation_ID(currentMessage.getConversation_ID());
//        negotiations.remove(currentMessage.getConversation_ID());
        goMsg_Dest(constant.MSG_Reject, msg, currentMessage.sender_hp);
    };
    
    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    
    protected abstract boolean checkRestrictions(int itemID);
    
    /**
     *  Description of the Method
     *
     *@param  currentMessage  Description of the Parameter
     *@return                 Description of the Return Value
     */
    
    protected synchronized boolean interpretCfp(Msg currentMessage) {
        String METHOD_NAME = "interpretCfp";
        boolean done = true;
        
        Integer factorID = new Integer(currentMessage.itemID);
        Integer conversation_ID = currentMessage.getConversation_ID();

        this.executionState = constant.NEGOTIATING;
        Msg replyMsg = null;
        //Check if this a tradeable item
        if (!tradeable.contains(new Integer(currentMessage.itemID))) {
            //Error, Reject
            if (isDebugEnabled()) { debug(currentMessage.itemID+" is not tradeable item."); };
            store.log(CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, currentMessage.itemID+" is not tradeable item.");
            replyMsg = currentMessage.createReply(ownID, constant.MSG_Reject, currentMessage.switchDocumentType(), 0);
            goMsg_Dest(constant.MSG_Reject, replyMsg, currentMessage.sender_hp);
            return false;
        };
        if ((!openCfPs.contains(currentMessage.request_ID)) && (currentMessage.itemID >= 2)) {
            //Error, Reject
            if (isDebugEnabled()) { debug(currentMessage.itemID+" is not requested."); };
            store.log(CLASS_NAME, constant.log_infos, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, currentMessage.itemID+" is not requested.");
            replyMsg = currentMessage.createReply(ownID, constant.MSG_Reject, currentMessage.switchDocumentType(), 0);
            goMsg_Dest(constant.MSG_Reject, replyMsg, currentMessage.sender_hp);
            return false;
        };

        if (isDebugEnabled()) { debug("Cfp cID received: " + currentMessage.getConversation_ID()); };
        
        if (negotiations.containsKey(conversation_ID)) {
            //do not interpret a second cfp in an ongoing conversation
            if (isDebugEnabled()) { debug("#" + conversation_ID.intValue() + ": second cfp."); };
            goMsg_Dest(constant.MSG_Reject,
            currentMessage.createReply(ownID, constant.MSG_Reject, currentMessage.switchDocumentType(), -1),
            currentMessage.sender_hp);
            Conversation currentConversation = (Conversation) negotiations.get(conversation_ID);
            currentConversation.setLastSentMessage(replyMsg);
            currentConversation.setLastReceivedMessage(currentMessage);
            negotiations.put(conversation_ID, currentConversation);
            return false;
        }
        
        // INBOX-Stack
        if (isNegotiating()) {
            addPendingNegotiation(currentMessage);
            return false;
        }
        else {
//            setNegotiating(true);
            setNegotiating(conversation_ID.intValue());
        }
        
        double myNewPrice = strategy.bargain(factorID, currentMessage.switchDocumentType(), currentMessage.price, -1, conversation_ID);
        if (isDebugEnabled()) { debug("Agent " + ownID + "| Conv: " + conversation_ID.intValue() + ": myNewPrice in interpretCFP: " + myNewPrice); };
        //if (isDebugEnabled()) { debug("Agent " + ownID + "| Conv: " + conversation_ID.intValue() + ": Action recommended: " + strategy.getRecommendedAction()); };
        switch (strategy.getRecommendedAction()) {
            case constant.MSG_Proposal:
                replyMsg = currentMessage.createReply(ownID, constant.MSG_Proposal, currentMessage.switchDocumentType(), myNewPrice);
                break;
            case constant.MSG_Reject:
                replyMsg = currentMessage.createReply(ownID, constant.MSG_Reject, currentMessage.switchDocumentType(), myNewPrice);
                break;
            case constant.MSG_Accept:
                replyMsg = currentMessage.createReply(ownID, constant.MSG_Accept, currentMessage.switchDocumentType(), myNewPrice);
                break;
            /*
             *  default:
             *  replyMsg = currentMessage.createReply(ownID, constant.MSG_Reject, currentMessage.switchDocumentType(), myNewPrice);
             *  break;
             */
        }
        replyMsg.sender_hp=LOCALHP;
        if (isDebugEnabled()) { debug("Agent " + ownID + "| Conv: " + conversation_ID.intValue() + ": replyMsg = " + replyMsg + ", curMsg= " + currentMessage); };
        store.log(CLASS_NAME, constant.log_business, METHOD_NAME, ownID, this.LOCALHP.p, conversation_ID, "Agent "+ownID+": CfP received, proposing "+myNewPrice);
        Conversation currentConversation=new Conversation(conversation_ID);
        currentConversation.setLastSentMessage(replyMsg);
        currentConversation.setLastReceivedMessage(currentMessage);
        negotiations.put(conversation_ID, currentConversation);
        goMsg_Dest(replyMsg.verdict, replyMsg, replyMsg.receiver_hp);
        
        
        // INBOX-Stack
        if (replyMsg.verdict == constant.MSG_Reject) {
            setNegotiating(false);
            if (anyPendingNegotiation()) {
                interpretCfp(getNextPendingNegotiation());
            } else {
                idleComputing();
            }
        }
        else if (replyMsg.verdict == constant.MSG_Accept)
            postAcceptanceMethod(currentMessage);
        //if (negotiations.containsKey(conversation_ID)) {if (isDebugEnabled()) { debug("sss2"); };};
        return done;
    }
    
    public void interpretinformResourceHost(Msg message) {
    }
    
    /** Getter for property money_balance.
     * @return Value of property money_balance.
     */
    public double getMoney_balance() {
        return this.money_balance;
    }
    
    /** Setter for property money_balance.
     * @param money_balance New value of property money_balance.
     */
    public void setMoney_balance(double money_balance) {
        this.money_balance=money_balance;
    }
    
    protected boolean decreaseBudget(double amount) {
        this.money_balance=this.money_balance-amount;
        if (this.money_balance >= 0) {
            return true;
        }
        else {
            return false;
        }
    };
    
    protected boolean increaseBudget(double amount) {
        this.money_balance=this.money_balance+amount;
        return true;
    };
    
    public void do_Commit(Conversation currentConversation, int SC_itemID) {
        do_Commit(currentConversation, currentConversation.getCorrespondingConversation(), SC_itemID);
    }
    
    public void do_Commit(Conversation currentConversation, Integer requestID, int SC_itemID) {
        
        Msg LastMessage= (Msg) currentConversation.getLastReceivedMessage();
        Msg msg = new Msg();
        Conversation initialConversation = (Conversation) negotiations.get(currentConversation.getCorrespondingConversation());
        msg.resource = initialConversation.getLastReceivedMessage().sender_hp;
        msg.sender_hp = this.LOCALHP;
        msg.receiver_hp = LastMessage.sender_hp;
        msg.price = LastMessage.price;
        msg.hop_counter = currentConversation.getDistance();
        msg.setConversation_ID(currentConversation.getConversationID());
        msg.request_ID = requestID;
        msg.itemID = LastMessage.itemID;
msg.SC_itemID = SC_itemID;
        currentConversation.setLastSentMessage(msg);
        msg.itemID = LastMessage.itemID;
        negotiations.put(currentConversation.getConversationID(), currentConversation);
        goMsg_Dest(constant.MSG_Confirm, msg, msg.receiver_hp);
    }
    
    protected abstract void interpretConfirm(Msg currentMessage);
    
    /**
     *  Description of the Method
     *
     *@param  CLASS_NAME   Description of the Parameter
     *@param  level        Description of the Parameter
     *@param  METHOD_NAME  Description of the Parameter
     *@param  Message      Description of the Parameter
     */
    
    protected abstract void interpretMoneyTransfer(Msg currentMessage);
    
    /**  interpretProvideService serves the purpose of measuring the path from SC
     *   to the client, but nothing else.
     *
     * @return    Description of the Return Value
     */
    protected void interpretProvideService(Msg currentMessage) {
        if (isDebugEnabled()) { debug("interpretProvideService on Agent_Source"); };
    }
    
    protected void interpret_BroadcastMessage(int Type, Msg currentMessage) {
        if (isDebugEnabled()) { debug("in interpret_BroadcastMessageQ"); };
        if (isDebugEnabled()) { debug("BroadcastMessage !!!!!!!! to "+currentMessage.receiver_hp.h+" from "+ currentMessage.sender_hp.h); };
        /* goMsg_Dest(constant.MSG_RequestService, currentMessage, currentMessage.receiver_hp);  */
    }
    
    protected void pay_tax(int transactionid_, double timeout_) {
        double now_ = getTime();
        double time_ = now_ + timeout_;
        TM_EVT deallocationtimeoutEvent = null;
        deallocationtimeoutEvent = new TM_EVT(constant.DEALLOC_SC_BW_EVT, transactionid_, time_);
        //if (constant.debuglog) debug("servicecopy "+this+": timeout set at  "+time_);
        setTimeoutAt(deallocationtimeoutEvent, time_);
    };
    
    public void start_periodical_services(double timeout) {
        periodicalServicesTimeout = timeout;
        
        double now_ = getTime();
        double time_ = now_ + timeout;
        TM_EVT  pay_taxevent= new TM_EVT(constant.PAY_TAX, ownID, time_);
        if (isDebugEnabled()) { debug("Start periodcal services"); };
        //debug("agent "+ownID+": timeout set at "+time_+" to pay tax.");
        
        setTimeoutAt(pay_taxevent, time_);
    };
    
    protected synchronized void timeout(Object evt_) {
        int resourceId = ((TM_EVT)evt_).resourceid;
        double timeout = ((TM_EVT)evt_).timeout;
        //debug("agent "+resourceId+": timeout  "+timeout);
        
        if (constant.stopsimulation)
            return;
        
        // Check if timeout was being reset or cancelled
        if (Double.isNaN(timeout))
            return;
        // XX: rounding error if in real-time simulation?
        if (timeout > getTime()) {
            setTimeoutAt(evt_, timeout);
            return;
        }
        
        switch(((TM_EVT)evt_).type) {
            case constant.PAY_TAX: 
                if (isDebugEnabled()) { debug("TO1"); };
                if (decreaseBudget(constant.periodically_tax)) {
                    if (isDebugEnabled()) { debug("Periodical Tax of "+constant.periodically_tax+" is paid"); };
                    //Initiate next event
                    double now_ = getTime();
                    double time_ = now_ + periodicalServicesTimeout;
                    TM_EVT pay_taxevent= new TM_EVT(constant.PAY_TAX, ownID, periodicalServicesTimeout);
                    if (isDebugEnabled()) { debug("Agent "+ownID+": next event set at "+time_); };
                    setTimeoutAt(pay_taxevent, time_);
                } else {
                    if (isDebugEnabled()) { debug("No money left."); };
                };
                break;
            case constant.START_DEMAND:
                if (isNegotiating()) {
                    ableToStartDemand = true;
                }
                else {
                    if (openCfPs.contains(new Integer(resourceId))) {
                        debug(" Demand " + resourceId + " not served");
                        debug("\tGraph output - confirm\t" + resourceId);
                        // Si no tiene el Cfp sabemos que ha habido un accept, sino no lo habria
                        // quitado, por lo tanto ya ha llamado a treatNextDemand
                        treatNextDemand();
                    }
                }
                break;
            case constant.NEXT_DEMAND:
                service_Demand(resourceId);
                break;
        }
    }
    
    public void treatNextDemand() {};
    public synchronized void service_Demand(int serviceID) {};
    
    public abstract void interpret_Plumage(Msg message);

    public void interpretStatus(Msg message) {}
    
    // INBOX-Stack
    public synchronized void rejectPendingNegotiations(Integer requestID) {
        String METHOD_NAME="rejectPendingNegotiations";
        while (anyPendingNegotiation()) {
            Msg currMsg = getNextPendingNegotiation();
            Msg rejMsg = new Msg();
            rejMsg.setConversation_ID(currMsg.getConversation_ID());
            rejMsg.request_ID = requestID;
            rejMsg.itemID = currMsg.itemID;
            rejMsg.sender_hp = this.LOCALHP;
            rejMsg.receiver_hp = currMsg.sender_hp;
            store.log("Agent_Source", constant.log_infos, "rejectPendingNegotiations", ownID, this.LOCALHP.p, currMsg.getConversation_ID(), "Reject send to " + rejMsg.receiver_hp.h + ": " + rejMsg.receiver_hp.p + " because of a former accept.");
            goMsg_Dest(constant.MSG_Reject, rejMsg, rejMsg.receiver_hp);
            
        }
    }
    
    /* Pending queue of CFps functions */
    public boolean isNegotiating()
    {
        return (negotiating >= 0) ;
    }
    
    public boolean isNegotiatingWith(int currNegotiation)
    {
        return (negotiating == currNegotiation);
    }
    
    public void setNegotiating(boolean state)
    {
        if (state) {
            negotiating = 0;
        } else {
            negotiating = -1;
        }
    }
    
    public void setNegotiating(int negotiation)
    {
        negotiating = negotiation;
    }
    
    public Msg getNextPendingNegotiation()
    {
        Msg tmpMsg = (Msg) pendingNegotiations.getFirst();
        pendingNegotiations.removeFirst();
        return tmpMsg;
    }
    
    public boolean anyPendingNegotiation()
    {
        return !pendingNegotiations.isEmpty();
    }
    
    public void addPendingNegotiation(Msg pendMsg)
    {
        pendingNegotiations.addLast(pendMsg);
    }

    public void removePendingNegotiations(int itemID)
    {
        ListIterator listIt;
        listIt = pendingNegotiations.listIterator(0);
        HashSet delElems = new HashSet();
        
        Msg currMsg;
        while (listIt.hasNext()) {
            currMsg = (Msg) listIt.next();
            if (currMsg.itemID == itemID)
                delElems.add(currMsg);
        }
        
        Iterator delIt = delElems.iterator();
        while (delIt.hasNext()) {
            pendingNegotiations.remove((Msg) delIt.next());
        }
    }

    public void removePendingNegotiations()
    {
        pendingNegotiations.clear();
    }

    public void idleComputing() {
    }
}
