package catnet.learning;

import java.util.Collection;
import java.rmi.RemoteException;

/**
 * TODO: Insert description of the class here.....
 *
 * @author     eymann <br>
 *             Jrg Meyer <a href="mailto:jomeyer@informatik.uni-freiburg.de">
 * @created    29. Juli 2002
 */
public interface ILearning {

	// actions recommended by the learning module

	/**
	 *  genotype should not be changed
	 */
	public static final int NONE = 0x0000;

	/**
	 *  inidicates if learning module recommends to adapt the genotype
	 *  according to the chosen learning procedure
	 */
	public static final int ADAPT = 0x0001;

	/**
	 *  send plumage to related agents
	 */
	public static final int PROPAGATE = 0x0002;

	/**
	 *  action text
	 */
	public final static String[] actionText = {"NONE", "ADAPT", "PROPAGATE", "PROPAGATE and ADAPT"};

	/**
	 * number of genes
	 */
	public static final int NUMBER_OF_GENES = 6;

	/**
	 *  Action to take recommended by the learning strategy. Constant values
	 *  could be 0 - do nothing 1 - adapt 2 - propagate
	 *
	 *@return    The recommendedAction value
	 */
	public int getRecommendedLearningAction();

	public float getAverageProfit();

	/**
	 *  This interface has to be implemented by every agent which changes its
	 *  Genotype by learning.
	 *
	 *@param  oldParameters  Description of the Parameter
	 *@return                The changed Genotype after processing the learning
	 *      mechanism
	 */
	public Object learn(Object oldParameters);


	/**
	 *  Adds a feature to the Information attribute of the ILearning object
	 *
	 *@param  o  The feature to be added to the Information attribute
	 */
	public void addPublicInformation(Object o);
	public void addPrivateInformation(Object o, Integer factorID);

	/**
	 *  Gets the plumage attribute of the ILearning object
	 *
	 *@param  fitness  Description of the Parameter
	 *@return          Description of the Return Value
	 */
	public Object createFitnessInformation(float fitness);


	/**
	 *  Gets the generation attribute of the ILearning object
	 *
	 *@return    The generation value
	 */
	public int getGeneration();

        public catnet.agent.Agent_Source getLearningOwner();
  
        public void setLearningOwner(catnet.agent.Agent_Source learningOwner);

}