package catnet.learning;

import catnet.learning.stdea.Plumage;
import catnet.util.RingArray;
import catnet.strategy.avalanche;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import catnet.agent.Agent_Source;

/**
 *
 * @author     Jrg Meyer <a href="mailto:jomeyer@informatik.uni-freiburg.de">
 * @created    9. September 2002
 */


public abstract class LearningTemplate implements ILearning{

	/**
	 *  Description of the Field
	 */
	protected Agent_Source learningOwner;

	/**
	 *  Generation counter so that the development of the strategy can be
	 *  followed
	 */
	protected int generationCounter = 0;

	/**
	 *  Gibt auf irgend eine Art an, wie viel der Agent schon agiert hat. Erst,
	 *  wenn er genug gehandelt hat, wird die fitness-Schaetzung relevant; also
	 *  soll er auch ein Weilchen warten, bis er anfaengt, sich zu vermehren.
	 *  -immi
	 *  Der Parameter gibt an, wieviele Verhandlungen der Agent schon abgeschlossen hat.
	 */
	protected int maturity = 0;

	/**
	 *  So hoch muss der maturity-Wert sein, bevor der Agent sich vermehren
	 *  darf. -immi
	 */
	protected int maturityThreshold = 10;

	/**
	 *  actions are encoded by bits
	 */
	protected int recommendedAction = ILearning.NONE;

	/**
	 *  profits with current genotype
	 */
	protected Hashtable profit = new Hashtable();

	// --- METHODS ----------------------------------------

	public void addPrivateInformation(Object o, Integer factorID) {
		maturity++;
		//learningOwner.printOutInfo("addPrivateInformation", "maturity: " + maturity);
		if (profit.containsKey(factorID)) {
			((RingArray)(profit.get(factorID))).addValue (((Float)o).doubleValue());
		}else{
			RingArray a = new RingArray (10);
			a.addValue(((Float)o).doubleValue());
			profit.put(factorID, a);
		}
	}

	public float getAverageProfit () {
		float sum = 0;
		Enumeration enum = profit.keys();
		while (enum.hasMoreElements()) {
			sum += (float) ((RingArray)(profit.get(enum.nextElement()))).getAverage();
		}
		return sum;
	}

	protected void newGenotype () {
		generationCounter++;
		maturity = 0;
		Enumeration enum = profit.keys();
		while (enum.hasMoreElements()) {
			((RingArray)(profit.get(enum.nextElement()))).clear();
		}
		recommendedAction = NONE;
	}

	/**
	 *  Erzeugt ein Plumage-Objekt passend zu diesem Agenten. -immi
	 *
	 *@param  profit  Description of the Parameter
	 *@return         Description of the Return Value
	 */
	public Object createFitnessInformation(float profit) {
		if ((recommendedAction & PROPAGATE) == PROPAGATE) {
		   recommendedAction -= PROPAGATE;
		}
                return new Plumage(new Float(profit).floatValue(), learningOwner.getStrategy().getMyGenotype());
	}

                /*return new Plumage(learningOwner.getAgentName(),
				learningOwner.getHoldingID(),
				(new Float(profit)).floatValue(),
				learningOwner.getStrategy().getMyGenotype());
                 */
	/**
	 *  Gets the recommendedAction attribute of the Smith98 object
	 *
	 *@return    The recommendedAction value
	 */
	public int getRecommendedLearningAction() {
		return recommendedAction;
	}

	public Agent_Source getLearningOwner() {
		return learningOwner;
	}

	/**
	 *  Sets the learningOwner attribute of the Smith98 object
	 *
	 *@param  learningOwner  The new learningOwner value
	 */
	public void setLearningOwner(Agent_Source learningOwner) {
		String METHOD_NAME = "setLearningOwner";
		this.learningOwner = learningOwner;
	}
	public int getGeneration() {
		return generationCounter;
	}

}