package catnet.learning.stdea;

import java.util.*;

/**
 *  A gene for float values.
 *
 *@author     RES, immi
 *            Jrg Meyer <a href="mailto:jomeyer@informatik.uni-freiburg.de">
 *@created    10. Oktober 2001
 *@version    1.1
 */
public class FloatGene extends Gene {

	// --- ATTRIBUTES -------------------------------------

	/**
	 * The width of the gaussian distribution, used wenn the value is randomized or mutated-
	 * variable multiplied with Gaussian random number when creep mutating
	 */
	float  gaussWidth = (float) 0.1;

	/**
	 * Minimum value of  the gene.
	 */
	float min = 0;

	/**
	 * Maximum value of the gene.
	 */
	float max = 1;

	// --- CONSTRUCTORS -----------------------------------

	public FloatGene(double val)
	{
		geneValue = new Float((float)val);
		min = 0;
		max = 1;
	}

	public FloatGene(float val)
	{
		geneValue = new Float(val);
		min = 0;
		max = 1;
	}

	public FloatGene(float val, float gb)
	{
		geneValue = new Float(val);
		min = 0;
		max = 1;
	}

	// --- METHODS -------------------------------------------

	// --- get / set -----------------------------------------

	public void setMin(float m)
	{   min = m;
	}

	public void setMax(float m)
	{   max = m;
	}

	public void setFloatValue(float f)
	{
		geneValue = new Float(f);
	}

	/**
	 * @return the value of this gene as float type variable
	 */
	public float getFloatValue()
	{
		return ((Float) getGeneValue()).floatValue();
	}

	/**
	 * @return the value of this gene as float type variable
	 */
	public double getDoubleValue()
	{
		return ((Float) getGeneValue()).doubleValue();
	}


	// --- necessary for genetic algorithm

	/**
	 * Initialize with random value with gaussian distribution.
	 */
	public void randomize(Random rand)
	{
		//geneValue = new Float(gaussWidth * rand.nextGaussian());
		geneValue = new Float (rand.nextFloat() * (max - min) + min);
	}

	/**
	 * "Creep" Mutation, which adds Gaussian noise to the geneValue
	 */
	public void mutate(Random rand)
	{
		// add a small random number to the gene, but don't raise above 1.0
		geneValue = new Float((((Float) geneValue).doubleValue() + gaussWidth * rand.nextGaussian()) % 1.0);
		testInterval();
	}

	/**
	 * If necessary, changes the value so that it lies within the interval [min,max].
	 */
	protected void testInterval()
	{
		if (getFloatValue() < min)
		{
			geneValue = new Float(min);
		}

		if (getFloatValue() > max)
		{
			geneValue = new Float(max);
		}
	}
}