package catnet.learning.stdea;

import java.util.*;
import java.io.*;

/**
 *  Here is a whole vector of genes. "DNA" would probably be a more appropriate
 *  name for this object, but transcribing our version of Egglets, this class
 *  stems from the Egglets genotype class.
 *
 *@author     Rob E. Smith, immi,
 *            Jrg Meyer <a href="mailto:jomeyer@informatik.uni-freiburg.de">
 *@created    7. Dezember 2001
 *@version    1.1
 */
public class Genotype extends Vector implements IMutateable, IRandomizeable, Cloneable, Serializable {

	// --- ATTRIBUTES -------------------------------------

	protected static Random rand = new Random();

	// --- CONSTRUCTORS -----------------------------------

	/**
	 *  Constructor for the Genotype object
	 *
	 *@param  p_acquisitiveness  Description of the Parameter
	 *@param  del_change         Description of the Parameter
	 *@param  del_jump           Description of the Parameter
	 *@param  p_satisfaction     Description of the Parameter
	 *@param  w_memory           Description of the Parameter
	 *@param  p_reputation       Description of the Parameter
	 */
	public Genotype(double p_acquisitiveness, double del_change, double del_jump, double p_satisfaction,
			double w_memory, double p_reputation) {
		/**
		 *  initializes a Genotype with six new values. A Genotype is a Vector
		 *  of four Doubles, the order indicates which one's which: 0 -
		 *  p_acquisitiveness, 1 - del_change, 2 - del_jump, 3 - p_satisfaction,
		 *  4 - w_memory 5 - p_reputaion To change values later, I recommend the
		 *  setElementAt( Object, int ) method of Vector.
		 */
		this.add(new FloatGene(p_acquisitiveness));
		this.add(new FloatGene(del_change));
		this.add(new FloatGene(del_jump));
		this.add(new FloatGene(p_satisfaction));
		this.add(new FloatGene(w_memory));
		this.add(new FloatGene(p_reputation));
	}


	/**
	 *  Constructor for the Genotype object
	 *
	 *@param  p_acquisitiveness  Description of the Parameter
	 *@param  del_change         Description of the Parameter
	 *@param  del_jump           Description of the Parameter
	 *@param  p_satisfaction     Description of the Parameter
	 *@param  w_memory           Description of the Parameter
	 */
	public Genotype(double p_acquisitiveness, double del_change, double del_jump, double p_satisfaction, double w_memory) {
		this(p_acquisitiveness, del_change, del_jump, p_satisfaction, w_memory, 1.0);
	}


	/**
	 *  Constructor for the Genotype object
	 *
	 *@param  p_acquisitiveness  Description of the Parameter
	 *@param  del_change         Description of the Parameter
	 *@param  del_jump           Description of the Parameter
	 *@param  p_satisfaction     Description of the Parameter
	 */
	public Genotype(double p_acquisitiveness, double del_change, double del_jump, double p_satisfaction) {
		this(p_acquisitiveness, del_change, del_jump, p_satisfaction, 0.65);
	}

	public Genotype (float[] g)
	{
		for (int i = 0; i < g.length; i++)
			this.add(new FloatGene(g[i]));
	}

	public Genotype (int length)
	{
		for (int i = 0; i < length; i++)
			add(new FloatGene(0));
		randomize(rand);
	}


	/**
	 *  Constructor for the Genotype object
	 */
	public Genotype() { }

	// --- METHODS -----------------------------------------

	/**
	 *  Same as elementAt(), but already returning a Gene class type.
	 *
	 *@param  p  Description of the Parameter
	 *@return    Description of the Return Value
	 */
	public Gene geneAt(int p) {
		return (Gene) elementAt(p);
	}

	/** Initialize everything with random values    */
	public final void randomize(Random rand)
	{
		for (int i = 0; i < size(); i++)
		{
			((Gene)elementAt(i)).randomize(rand);
		}
	}

	/**  Small mutation - each gene is mutated with probability 0.15    */
	public final void mutate(Random rand)
	{
		// mutate all genes
		for (int i = 0; i < size(); i++)
		{
			Gene g = (Gene) elementAt(i);
			if (rand.nextFloat() < 0.15)
			   g.mutate(rand);
		}
	}

	/**  mutation - each gene is mutated with probability p */
	public final void mutate(Random rand, double p)
	{
		for (int i = 0; i < size(); i++)
		{
			Gene g = (Gene) elementAt(i);
			if (rand.nextFloat() < p)
			   g.mutate(rand);
		}
	}

	/**
	 * Create a new gene by crossing with geneOfFather.
	 * @param geneOfFather should have the same length
	 */
	public Genotype cross (Genotype geneOfFather, Random Rand)
	{
		if (size() != geneOfFather.size())
		{
			System.out.println("genes of father and mother have different length "
									  + size() + " : " + geneOfFather.size());
			return this;
		}
		Genotype ret = new Genotype(size());
		for (int i = 0; i < size(); i++)
		{
			FloatGene gf = (FloatGene) geneOfFather.elementAt(i);
			FloatGene gm = (FloatGene) elementAt(i);
			if (Rand.nextFloat() < 0.5)
				ret.setElementAt(new FloatGene(gf.getFloatValue()), i);
			else
				ret.setElementAt(new FloatGene(gm.getFloatValue()), i);
		}
		return ret;
	}

	/**
	 * Create a new gene by crossing with geneOfFather.
	 * @param geneOfFather should have the same length
	 * @param p probability of gene of father to be chosen
	 *          in crossing procedure
	 */
	public Genotype cross (Genotype geneOfFather, Random Rand, double p)
	{
		if (size() != geneOfFather.size())
		{
			System.out.println("genes of father and mother have different length "
									  + size() + " : " + geneOfFather.size());
			return this;
		}
		Genotype ret = new Genotype(size());
		for (int i = 0; i < size(); i++)
		{
			FloatGene gf = (FloatGene) geneOfFather.elementAt(i);
			FloatGene gm = (FloatGene) elementAt(i);
			if (Rand.nextFloat() < p)
				ret.setElementAt(new FloatGene(gf.getFloatValue()), i);
			else
				ret.setElementAt(new FloatGene(gm.getFloatValue()), i);
		}
		return ret;
	}

	public float[] toFloatArray()
	{
		float[] array = new float[size()];
		for (int i = 0; i < array.length; i++)
			array[i] = ((FloatGene)geneAt(i)).getFloatValue();
		return array;
	}


	/**
	 *  Output of the whole genotype as a string
	 *
	 *@return    Description of the Return Value
	 */
	public String toString() {
		/*
		 *  ret += "p_acq:\t"+ Math.round(1000 * ((FloatGene)elementAt(0)).getDoubleValue()) / 1000.0;
		 *  ret += "\tdel_ch:\t"+ Math.round(1000 * ((FloatGene)elementAt(1)).getDoubleValue()) / 1000.0;
		 *  ret += "\tdel_jp:\t"+ Math.round(1000 * ((FloatGene)elementAt(2)).getDoubleValue()) / 1000.0;
		 *  ret += "\tp_sat:\t" + Math.round(1000 * ((FloatGene)elementAt(3)).getDoubleValue()) / 1000.0;
		 *  ret += "\tw_mem:\t" + Math.round(1000 * ((FloatGene)elementAt(4)).getDoubleValue()) / 1000.0;
		 *  ret += "\tp_rep\t" + Math.round(1000 * ((FloatGene)elementAt(5)).getDoubleValue()) / 1000.0;
		 */
		String ret = "(" + Math.round(1000 * ((FloatGene) elementAt(0)).getDoubleValue()) / 1000.0;
		ret += ", " + Math.round(1000 * ((FloatGene) elementAt(1)).getDoubleValue()) / 1000.0;
		ret += ", " + Math.round(1000 * ((FloatGene) elementAt(2)).getDoubleValue()) / 1000.0;
		ret += ", " + Math.round(1000 * ((FloatGene) elementAt(3)).getDoubleValue()) / 1000.0;
		ret += ", " + Math.round(1000 * ((FloatGene) elementAt(4)).getDoubleValue()) / 1000.0;
		ret += ", " + Math.round(1000 * ((FloatGene) elementAt(5)).getDoubleValue()) / 1000.0;
		ret += ")";
		return ret;
	}


	/**
	 *  Set single values, used for changing estimations in the partnerlist of
	 *  the agents
	 *
	 *@param  new_acq  The new acquisitiveness value
	 */
	public final void setAcquisitiveness(double new_acq) {
		setElementAt(new FloatGene(new_acq), 0);
	}


	/**
	 *  Gets the acquisitiveness attribute of the Genotype object
	 *
	 *@return    The acquisitiveness value
	 */
	public final double getAcquisitiveness() {
		return ((FloatGene) elementAt(0)).getDoubleValue();
	}


	/**
	 *  Sets the deltaChange attribute of the Genotype object
	 *
	 *@param  new_dchg  The new deltaChange value
	 */
	public final void setDeltaChange(double new_dchg) {
		setElementAt(new FloatGene(new_dchg), 1);
	}


	/**
	 *  Gets the deltaChange attribute of the Genotype object
	 *
	 *@return    The deltaChange value
	 */
	public final double getDeltaChange() {
		return ((FloatGene) elementAt(1)).getDoubleValue();
	}


	/**
	 *  Sets the deltaJump attribute of the Genotype object
	 *
	 *@param  new_djmp  The new deltaJump value
	 */
	public final void setDeltaJump(double new_djmp) {
		setElementAt(new FloatGene(new_djmp), 2);
	}


	/**
	 *  Gets the deltaJump attribute of the Genotype object
	 *
	 *@return    The deltaJump value
	 */
	public final double getDeltaJump() {
		return ((FloatGene) elementAt(2)).getDoubleValue();
	}


	/**
	 *  Sets the satisfaction attribute of the Genotype object
	 *
	 *@param  new_sat  The new satisfaction value
	 */
	public final void setSatisfaction(double new_sat) {
		setElementAt(new FloatGene(new_sat), 3);
	}


	/**
	 *  Gets the satisfaction attribute of the Genotype object
	 *
	 *@return    The satisfaction value
	 */
	public final double getSatisfaction() {
		return ((FloatGene) elementAt(3)).getDoubleValue();
	}


	/**
	 *  Sets the weightMemory attribute of the Genotype object
	 *
	 *@param  new_wmem  The new weightMemory value
	 */
	public final void setWeightMemory(double new_wmem) {
		setElementAt(new FloatGene(new_wmem), 4);
	}


	/**
	 *  Gets the weightMemory attribute of the Genotype object
	 *
	 *@return    The weightMemory value
	 */
	public final double getWeightMemory() {
		return ((FloatGene) elementAt(4)).getDoubleValue();
	}


	/**
	 *  Sets the reputation attribute of the Genotype object
	 *
	 *@param  new_rep  The new reputation value
	 */
	public final void setReputation(double new_rep) {
		setElementAt(new FloatGene(new_rep), 5);
	}


	/**
	 *  Gets the reputation attribute of the Genotype object
	 *
	 *@return    The reputation value
	 */
	public final double getReputation() {
		return ((FloatGene) elementAt(5)).getDoubleValue();
	}
}
