package catnet.learning.stdea;

/**
 *  <p>
 *
 *  English Description: <p>
 *
 *  This class Plumage holds <em> everything </em> needed for reproduction of
 *  the agents. If an agent wants to mate, it sends a plumage object to other
 *  agents. It also collects all plumage objects it receives. If the agents has
 *  collected enough agents (more than a specified threshold, it chooses the
 *  best of these. If the agent chooses a particular plumage object, it takes
 *  the information in that object and inherits another agent. This Plumage
 *  class is a bit different from those given in the Egglets, since it contains
 *  everything needed for reproductions. To reproduce, an agent only needs the
 *  other's plumage, not the other agent itself, as with the Egglets. <p>
 *
 *  German Description: <p>
 *
 *  Plumage enthaelt alle Paarungs-Informationen. Wenn ein Agent sich paaren
 *  will, sendet er ein Plumage-Objekt an andere Agenten. Ausserdem sammelt er
 *  selbst Plumage-Objekte, die er bekommt. Wenn ihm eins gut genug gefaellt,
 *  nimmt er die darin enthaltenen Erb-Informationen (Genotype) und erzeugt
 *  damit Nachwuchs. "Gut genug gefaellt" ist zur Zeit definiert als: Wenn er
 *  genug Plumages gesammelt hat, sucht er sich das beste aus. Das Plumage hier
 *  unterscheidet sich etwas von dem bei Robert E. Smith: Hier enthaelt es <em>
 *  alles</em> , was zum Fortpflanzen noetig ist. Ein Agent braucht, um sich mit
 *  einem anderen Agenten fortzupflanzen nur dessen Plumage. Bei den Egglets
 *  unterhalten sich die beiden Agenten noch, bevor sie Nachkommen erzeugen.
 *
 *@author     RES, Immi
 *@created    10. Oktober 2001
 */
public class Plumage implements java.io.Serializable {
    /**
     *  the name of the sender
     */
    protected String name;

    /**
     *  the tribe of the sender
     */

    /**
     *  the tribe of the sender the fitness of the sender
     */
    public float fitness;
    
    protected Integer itemID;

    /**
     *  the gene information of the sender
     */
    protected Genotype senderGenes;

    /**
     *  Description of the Field
     */
    private String tribe;


    /**
     *  create a new plumage from name, mate_group, fitness, and sender's genes.
     *
     *@param  na   Description of the Parameter
     *@param  sta  Description of the Parameter
     *@param  fit  Description of the Parameter
     *@param  erb  Description of the Parameter
     */
    public Plumage(float fit, Genotype erb) {
        
        fitness = fit;
        senderGenes = erb;
    }


    /**
     *@return    name of the sender
     */
    public String getName() {
        return name;
    }


    /**
     *@return    tribe of the sender
     */

    /**
     *@return    tribe of the sender
     *@return    fitness of the sender as estimated by himself
     */
    public float getFitness() {
        return fitness;
    }


    /**
     *@return    genes of the sender
     */
    public Genotype getSenderGenes() {
        return senderGenes;
    }


    /**
     *  Gets the tribe attribute of the Plumage object
     *
     *@return    The tribe value
     */
    public String getTribe() {
        return tribe;
    }
}
