package catnet.learning.stdea;

import java.util.Vector;
import java.util.Enumeration;
import catnet.learning.ILearning;
import catnet.learning.LearningTemplate;
import catnet.agent.Agent_Source;

/**
 *  berschrift: B2B-OS Beschreibung: Copyright: Copyright (c) 2001
 *  Organisation: IIG Telematik, Albert-Ludwigs-Universitaet Freiburg
 *
 *@author     Dr. Torsten Eymann,
 *            Jrg Meyer <a href="mailto:jomeyer@informatik.uni-freiburg.de">
 *@created    1. Januar 2002
 *@version    1.1
 */
public class Smith98 extends LearningTemplate {
	/**
	 *  Constructor for the Smith98 object
	 */
	public Smith98() { }


	/**
	 *  Description of the Field
	 */
	private String CLASS_NAME = "Smith98";

	/**
	 *  This variable holds the name of the agent's first parent.
	 */
	protected String mother;

	/**
	 *  This variable holds the name of the agent's second parent.
	 */
	protected String father;

	/**
	 *  Controls mating behavior of agents. Only within the same mate_group will
	 *  agents exchange plumage objects. mate_group should either be initialized
	 *  as the tribe or the agentType in the constructor. Controlls mating
	 *  behavior of agents. Only within the same mate_group will agents exchange
	 *  plumage objects. mate_group should either be initialized as the tribe or
	 *  the agentType in the constructor. Number of plumages the agent has to
	 *  collect prior to the reproduction act <p>
	 *
	 *  Anzahl der Plumages, die der Agent sammelt, bevor er sich vermehrt -immi
	 */
	protected int courterThreshold = 10;

	/**
	 *  This vector holds the plumages that are sent from other agents and
	 *  collected by this agent . When the agents holds enough plumages (see
	 *  <code>courterThreshold</code>), it selects the best of these and
	 *  reproduces with it. All access to courterPlumages should be synchonized
	 *  on itself.<p>
	 *
	 *  Die Plumages, die der Agent gesammelt hat. Der Agent sammelt Plumages,
	 *  die er von anderen Agenten bekommt. Wenn er genug hat, sucht er sich den
	 *  besten aus und vermehrt sich damit. -immi
	 *
	 *@see    #courterThreshold
	 */
	protected Vector courterPlumages = new Vector();

	/**
	 *  This variable serves as a switch for having an unbounded reproduction or
	 *  a replacement reproduction. If <code>dieForChild</code> is true, then
	 *  this parent agent dies after having reproduced. The child replaces the
	 *  parent and the population size is kept constant. If <code>dieForChild</code>
	 *  is false, both parent and child run alongside. This effectively doubles
	 *  this successful strategy and fuels the markov chain property of the
	 *  system. <p>
	 *
	 *  Wenn der Agent stirbt (agentDoomed = true): true, wenn der Grund der
	 *  ist, dass der Agent ein Kind gekriegt hat. Das ist eine Methode, die
	 *  Groesse der Bevoelkerung zu regulieren. (Jedes mal, wenn ein Agent
	 *  geboren wird, stirbt auch einer.) Wird nicht wirklich benoetigt, aber
	 *  vielleicht will man wissen, warum die Agenten sterben. -immi
	 */
	protected boolean dieForChild = false;

	/**
	 *  if this variable is true, the agent will be immortal and the EA is used
	 *  to update its genotype with a crossover between its own genotype and the
	 *  best plumage received.
	 */
	protected boolean immortality = false;

	/**
	 *  switch to turn mating on(true) and off(false)
	 */
	private boolean reproduce = true;

	/**
	 *  Entscheidet, wann und mit wem der Agent sich vermehren will. In dieser
	 *  Version wird einfach geprueft, ob dieser Agent schon genug Plumages
	 *  gesammelt hat (mindestens courterThreshold viele). Wenn ja, wird das
	 *  Plumage-Objekt mit dem hoechsten Fitness-Wert zurueckgeliefert
	 *
	 *@return    null wenn der Agent sich im Moment nicht vermehren will, sonst
	 *      das Plumage-Objekt von dem Agenten, mit dem er sich vermehren will
	 */
	protected Plumage getMate() {
		Plumage best = null;
		Vector cp = null;
		synchronized (courterPlumages) {
			cp = (Vector) courterPlumages.clone();
		}
		// StringBuffer sb = new StringBuffer("court: ");
		for (Enumeration e = cp.elements(); e.hasMoreElements(); ) {
			Plumage p = (Plumage) e.nextElement();
			if (best == null) {
				best = p;
			} else if (p.getFitness() > best.getFitness()) {
				best = p;
				// sb.append(Math.round(100 * p.getFitness()));
				// sb.append(":");
			}
		}
		// sb.append("--"+best.getFitness());
		// trace(sb.toString());
		return best;
	}


	/**
	 *  Description of the Field
	 */
	private int plumCount = 0;

	/**
	 *  Description of the Field
	 */
	private int plumFreq = 0;


	/**
	 *  Tut, was zur Erhaltung der Art regelmaessig getan werden muss. D.h.:
	 *  Plumages an andere Agenten schicken und pruefen, ob man selbst so viele
	 *  Plumages bekommen hat, dass man sich vermehren will. Die
	 *  Plumage-Messages werden von AvLocationAgent zufaellig an einen anderen
	 *  Agenten (in der gleichen Location) weitergeleitet. Nur, wenn der Agent
	 *  schon genug gehandelt hat und der Fitness-Wert eine relevante Bedeutung
	 *  hat, darf der Agent versuchen, sich vermehren. -Immi
	 *
	 *@param  oldGenotype  Description of the Parameter
	 *@return              Description of the Return Value
	 */
	public Object learn(Object oldGenotype) {
		Plumage p = getMate();
		// get the "best" partner from list
		if (p != null) {
			// cross own genes with partner's genes
                        Genotype g = learningOwner.getStrategy().getMyGenotype().cross(p.getSenderGenes(), new java.util.Random());
			g.mutate(new java.util.Random());
			newGenotype();
			return g;
		}
		// no partner found, repeat old genotype
		/* this is arguable.  this way, reproduction esp. plumages are slowed down
		newGenotype(); */
		// infoServer.logPopulationGenes(genes, type, -1)
		return oldGenotype;
	}

	protected void newGenotype()
	{
		super.newGenotype();
		maturity = 0;
		synchronized (courterPlumages) {
			courterPlumages.clear();
		}
	}

        public void setLearningOwner(Agent_Source learningOwner) {
           this.learningOwner = learningOwner;
        }
	/**
	 *  Adds a feature to the Information attribute of the Smith98 object
	 *
	 *@param  o  The feature to be added to the Information attribute
	 */
	public void addPublicInformation(Object o) {
		courterPlumages.addElement((Plumage) o);
		if (courterPlumages.size() >= courterThreshold && maturity >= maturityThreshold) {
				recommendedAction |= PROPAGATE;
				recommendedAction |= ADAPT;
		}
	}

	public void addPrivateInformation(Object o, Integer factorID) {
		super.addPrivateInformation(o, factorID);
		if (maturity % maturityThreshold == 0) {
			recommendedAction |= PROPAGATE;
			if (courterPlumages.size() >= courterThreshold) {
				recommendedAction |= ADAPT;
			}
		}
	}
}
