package catnet.negotiation;

import edu.uah.math.distributions.Data;
import edu.uah.math.distributions.NormalDistribution;
import catnet.data.constant;

import catnet.util.*;

/**
 *  berschrift: B2B-OS Beschreibung: Copyright: Copyright (c) 2001
 *  Organisation: IIG Telematik, Albert-Ludwigs-Universitaet Freiburg
 *
 *@author     Dr. Torsten Eymann
 *@created    18. Dezember 2001
 *@version    1.0
 */

public class PriceDistribution {

    /**
     *  Description of the Field
     */
    protected RingArray priceData;

    /**
     *  Description of the Field
     */
    /**
     *  Description of the Field
     */
    private double value;

    /**
     *  Description of the Field
     */
    private double lastAgreementPrice = -1;

    /**
     *  Description of the Field
     */
    private double upperLimitPrice = -1;
    /**
     *  Description of the Field
     */
    private double lowerLimitPrice = -1;
    /**
     *  Description of the Field
     */

    /**
     *  Description of the Field
     */
    private double average = -1;


    /**
     *  Constructor for the PriceDistribution object
     *
     *@param  value  Description of the Parameter
     *@param  size   Description of the Parameter
     */
    public PriceDistribution(double value, int size) {
        priceData = new RingArray(size);
        this.addValue(value);
        this.setUpperLimitPrice(value);
        this.setLowerLimitPrice(value);
        this.setLastAgreementPrice(value);
    }


    /**
     *  Constructor for the PriceDistribution object
     *
     *@param  a    Description of the Parameter
     *@param  v    Description of the Parameter
     *@param  min  Description of the Parameter
     *@param  max  Description of the Parameter
     */
    public PriceDistribution(double a, double v, double min, double max) {
        priceData = new RingArray();
        this.addValue(a);
        this.setUpperLimitPrice(Math.max(min, max));
        this.setLowerLimitPrice(Math.min(min, max));
        this.setLastAgreementPrice(a);
    }


    /**
     *  Description of the Field
     *
     *@return    The lastAgreementPrice value
     */

    public double getLastAgreementPrice() {
        return lastAgreementPrice;
    }


    /**
     *  Sets the lastAgreementPrice attribute of the PriceDistribution object
     *
     *@param  lastAgreementPrice  The new lastAgreementPrice value
     */
    public void setLastAgreementPrice(double lastAgreementPrice) {
        this.lastAgreementPrice = lastAgreementPrice;
    }


    /**
     *  Gets the lowerLimitPrice attribute of the PriceDistribution object
     *
     *@return    The lowerLimitPrice value
     */
    public double getLowerLimitPrice() {
        return lowerLimitPrice;
    }


    /**
     *  Sets the lowerLimitPrice attribute of the PriceDistribution object
     *
     *@param  lowerLimitPrice  The new lowerLimitPrice value
     */
    public void setLowerLimitPrice(double lowerLimitPrice) {
        this.lowerLimitPrice = lowerLimitPrice;
    }


    /**
     *  Sets the upperLimitPrice attribute of the PriceDistribution object
     *
     *@param  upperLimitPrice  The new upperLimitPrice value
     */
    public void setUpperLimitPrice(double upperLimitPrice) {
        this.upperLimitPrice = upperLimitPrice;
    }


    /**
     *  Sets the initialPrice attribute of the PriceDistribution object
     *
     *@param  price  The new initialPrice value
     *@param  type   The new initialPrice value
     */
    public void setInitialPrice(double price, int type) {
        switch (type) {
            case constant.ASK:
                this.lowerLimitPrice = price;
                break;
            case constant.BID:
                this.upperLimitPrice = price;
                break;
            default:
                //error
                break;
        }
    }


    /**
     *  Sets the limitPrice attribute of the PriceDistribution object
     *
     *@param  price  The new limitPrice value
     *@param  type   The new limitPrice value
     */
    public void setLimitPrice(double price, int type) {
        switch (type) {
            case Proposal.ASK:
                this.upperLimitPrice = price;
                break;
            case Proposal.BID:
                this.lowerLimitPrice = price;
                break;
            default:
                //error
                break;
        }
    }


    /**
     *  Gets the upperLimitPrice attribute of the PriceDistribution object
     *
     *@return    The upperLimitPrice value
     */
    public double getUpperLimitPrice() {
        return upperLimitPrice;
    }


    /**
     *  Adds data to the circular ring-array and overwrites the least recent
     *  value if full.
     *
     *@param  value  The feature to be added to the Value attribute
     */
    public void addValue(double value) {
        priceData.addValue(value);
        if (weightedAverage == 0) {
            weightedAverage = value;
        } else {
            weightedAverage = (value * weight) + weightedAverage * (1 - weight);
        }
    }


    /**
     *  Adds data to the circular ring-array and overwrites the least recent
     *  value if full.
     *
     *@param  value   The feature to be added to the Value attribute
     *@param  weight  The feature to be added to the Value attribute
     */
    public synchronized void addValue(double value, double weight) {
        setWeight(weight);
        if (weightedAverage == 0) {
            weightedAverage = value;
        } else {
            weightedAverage = (value * weight) + weightedAverage * (1 - weight);
        }
        priceData.addValue(value);
    }


    /**
     *  Gets the average attribute of the PriceDistribution object
     *
     *@return    The average value
     */
    public double getMeanAverage() {
        return priceData.getMean();
    }


    /**
     *  Description of the Field
     */
    private double weight = 0.5;
    /**
     *  Description of the Field
     */
    private double weightedAverage = 0;


    /**
     *  Gets the weightedAverage attribute of the PriceDistribution object
     *
     *@return    The weightedAverage value
     */
    public double getWeightedAverage() {
        return weightedAverage;
    }


    /**
     *  Sets the weight attribute of the RingArray object
     *
     *@param  weight  The new weight value
     */
    public void setWeight(double weight) {
        this.weight = weight;
    }


    /**
     *  Gets the weight attribute of the RingArray object
     *
     *@return    The weight value
     */
    public double getWeight() {
        return weight;
    }


    /**
     *  Gets the maximum attribute of the PriceDistribution object
     *
     *@return    The maximum value
     */
    public double getMaximum() {
        return priceData.getMaximum();
    }


    /**
     *  Gets the minimum attribute of the PriceDistribution object
     *
     *@return    The minimum value
     */
    public double getMinimum() {
        return priceData.getMinimum();
    }


    /**
     *  Gets the distribution attribute of the PriceDistribution object
     *
     *@return    The distribution value
     */
    public double getDistribution() {
        NormalDistribution d = new NormalDistribution(priceData.getMean(), Math.sqrt(priceData.getVariance()));
        return d.simulate();
    }

}
