package catnet.strategy;

import java.util.HashMap;
import catnet.learning.stdea.Genotype;
import catnet.learning.ILearning;
import catnet.negotiation.NegotiationHistory;
import catnet.agent.Agent_Source;

/**
 *  berschrift: B2B-OS Beschreibung: Copyright: Copyright (c) 2001
 *  Organisation: IIG Telematik, Albert-Ludwigs-Universitaet Freiburg
 *
 *@author     Dr. Torsten Eymann
 *@created    11. Oktober 2001
 *@version    1.0
 */

public interface IPricing {
    /**
     *  This method should handle all computation related to finding a price for
     *  distributive negotiation. Basically, buyers and sellers will use the
     *  same method and compare their different valuation of the good (unit
     *  price). This valuation changes over the course of the negotiation by
     *  making concessions. If both values match, a compromise has been found
     *  and the good will (probably) change its owner.
     *
     *@param  opponentsNewPrice       actual price how opponent values one unit
     *      of the good
     *@param  myOldPrice              last price that I valued one unit of the
     *      good
     *@param  factorID                Description of the Parameter
     *@param  negotiationID           Description of the Parameter
     *@param  proposalTypeToGenerate  Description of the Parameter
     *@return                         price to reply to opponent
     */
    public double bargain(Integer factorID, int proposalTypeToGenerate, double opponentsNewPrice, double myOldPrice, Integer negotiationID);


    /**
     *  Sets the parameters attribute of the IPricing object
     *
     *@param  strategyHash  The new parameters value
     */
    public void setParameters(HashMap strategyHash);

    public double getLowerLimitPrice(Integer factorID);

    public double getUpperLimitPrice(Integer factorID);
    /**
     *  Sets the prices attribute of the IPricing object
     *
     *@param  factorID  The new prices value
     *@param  min       The new prices value
     *@param  max       The new prices value
     */
    public void setPrices(Integer factorID, double min, double max);


    /**
     *  Gets the limitPrice attribute of the IPricing object, which means the
     *  price UP to which a good is ASKed or DOWN to which a good is BID for
     *  selling. (The initialPrice is technically the reverse).
     *
     *@param  factorID  Description of the Parameter
     *@param  type      Description of the Parameter
     *@return           The limitPrice value
     */
    public double getLimitPrice(Integer factorID, int type);


    /**
     *  Gets the negotiationHistory attribute of the IPricing object
     *
     *@param  conversationID  Description of the Parameter
     *@return                 The negotiationHistory value
     */
    public NegotiationHistory getNegotiationHistory(String conversationID);


    /**
     *  Gets the initialPrice attribute of the IPricing object, that means the
     *  price UP from which a good is BID or DOWN from which a good is ASKed.
     *  (The limitPrice is technically the reverse).
     *
     *@param  factorID  Description of the Parameter
     *@param  type      Description of the Parameter
     *@return           The initialPrice value
     */
    public double getInitialPrice(Integer factorID, int type);


    /**
     *  Gets the averagePrice attribute of the IPricing object
     *
     *@param  factorID  Description of the Parameter
     *@return           The averagePrice value
     */
    public double getAveragePrice(Integer factorID);


    /**
     *  Description of the Field
     */
    //static int REJECT = 0;
    /**
     *  Description of the Field
     */
    //static int ACCEPT = 1;
    /**
     *  Description of the Field
     */
    //static int PROPOSE = 2;

    /**
     *  Description of the Field
     */
    public final static String[] actionText = {"REJECT", "ACCEPT", "PROPOSE"};


    /**
     *  Action to take recommended by the pricing strategy. Constant values are
     *  as follows:
     *
     *@return    The recommendedAction value
     */

    public int getRecommendedAction();


    /**
     *  Gets the executionState attribute of the zeroIntelligence object
     *
     *@return    The executionState value
     */
    public String getExecutionState();

    public void setLearningModule(ILearning learningModule);

    public ILearning getLearningModule();
    
    public void setStrategyOwner(Agent_Source strategyOwner);
    
    public Agent_Source getStrategyOwner();
    /**
     *  Sets the myGenotype attribute of the IPricing object
     *
     *@param  myGenotype  The new myGenotype value
     */
    public void setMyGenotype(Genotype myGenotype);

    public void randomizeMyGenotype();
    /**
     *  Gets the myGenotype attribute of the IPricing object
     *
     *@return    The myGenotype value
     */
    public Genotype getMyGenotype();

}
