package catnet.strategy;

import catnet.learning.ILearning;
import catnet.accounting.Factor;
import catnet.learning.stdea.Genotype;
import catnet.negotiation.NegotiationHistory;
import catnet.negotiation.PriceDistribution;
import catnet.data.constant;
import catnet.util.Tool;
import catnet.agent.Agent_Source;
import edu.uah.math.distributions.*;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Random;

/**
 *  berschrift: B2B-OS Beschreibung: Copyright: Copyright (c) 2001
 *  Organisation: IIG Telematik, Albert-Ludwigs-Universitaet Freiburg
 *
 *@author     Dr. Torsten Eymann, Michael Reinicke
 *@created    20. Juni 2002
 *@version    1.0
 */

public class allaccept implements IPricing {

    /**
	 *  Description of the Field
	 */
    protected String CLASS_NAME = "allaccept";

    /**
	 *  Spread around the average market price
	 */
    protected int ACCEPTABLE_SPREAD = 4;

    public ILearning learning = new catnet.learning.stdea.Smith98();

    private Agent_Source strategyOwner;
    /**
	 *  The strategy of the agent contained in a Genotype object.
	 *
	 *@see    Genotype
	 */
    private Genotype myGenotype;

    /**
	 *  HashMap to hold my own limit and average prices. Key is factorID, Value
	 *  is a PriceDistribution object with the price for that good.
	 */
    private HashMap priceHash = new HashMap();

    /**
	 *  Small hashmap to hold the negotiation history. Key is the negotiation
	 *  ID, value a NegotationHistory object.
	 *
	 *@see    NegotationHistory
	 */
    private HashMap historyHash = new HashMap();

    /**
	 *  The type of action this strategy recommends to the pro-active module.
	 *
	 *@see    constant#Msg_Accept
	 *@see    constant#Msg_Reject
	 *@see    constant#Msg_Propose
	 */
//    private int recommendedAction = 0;

    /**
	 *  Description of the Field
	 */
    private String executionState = "";


    private Recommendation rec;

    /**
	 *  cf is the LOCALE currencyFormat to format prices in the logfiles
	 */
    private NumberFormat cf;


    /**
	 *  Description of the Field
	 */
    private RandomVariable RND = new RandomVariable(new ContinuousUniformDistribution());


    /**
	 *  Constructor for the allaccept object
	 */
    public allaccept() {
		cf = NumberFormat.getNumberInstance();
		//principal = null;
    }

    /**
	 *  Sets the parameters attribute of the allaccept object
	 *
	 *@param  strategy_blueprint  The new parameters value
	 */
    public synchronized void setParameters(HashMap strategy_blueprint) {
		String METHOD_NAME = "setParameters()";

		try {
			this.myGenotype = new Genotype(
				new ContinuousUniformDistribution(
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "acquisitiveness/average")) -
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "acquisitiveness/variance")),
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "acquisitiveness/average")) +
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "acquisitiveness/variance"))).simulate(),
				new ContinuousUniformDistribution(
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_step/average")) -
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_step/variance")),
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_step/average")) +
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_step/variance"))).simulate(),
				new ContinuousUniformDistribution(
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_next/average")) -
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_next/variance")),
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_next/average")) +
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_next/variance"))).simulate(),
				new ContinuousUniformDistribution(
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "satisfaction/average")) -
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "satisfaction/variance")),
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "satisfaction/average")) +
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "satisfaction/variance"))).simulate(),
				new ContinuousUniformDistribution(
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "weight_memory/average")) -
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "weight_memory/variance")),
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "weight_memory/average")) +
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "weight_memory/variance"))).simulate(),
				new ContinuousUniformDistribution(
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "reputation/average")) -
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "reputation/variance")),
											  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "reputation/average")) +
												  Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "reputation/variance"))).simulate()
			);
		} catch (NullPointerException ex) {
			this.myGenotype = new Genotype(0.5, 0.5, 0.5, 0.7, 0.5, 1.0);
		}
		/*        principal.printOut(CLASS_NAME, Logger.INFO, METHOD_NAME,
		 "strategy parameters = "
		 + myGenotype);
		 */
    }

    public void randomizeMyGenotype() {
            Random rnd = new Random();
            //P_acq between 0.4 und 0.8
            //Delta_change between 0.1 and 0.4
            //Delta_jump between 0.1 and 0.6
            //P_sat between 0.4 and 0.8
            //W_mem between 0.3 and 0.7
            //P_rep always 1.0
            
            double p_acquisitiveness=rnd.nextInt(40);
            p_acquisitiveness = 0.4 + p_acquisitiveness /100;
            double del_change=rnd.nextInt(30);
            del_change = 0.1 + del_change/100;
            double del_jump=rnd.nextInt(50);
            del_jump = 0.1 + del_jump/100;
            double p_satisfaction=rnd.nextInt(40);
            p_satisfaction = 0.4 + p_satisfaction/100;
            double w_memory=rnd.nextInt(40);
            w_memory = 0.3 + w_memory/100;
            double p_reputation=1.0;        
            
            this.myGenotype = new Genotype(p_acquisitiveness, del_change, del_jump, p_satisfaction, w_memory, p_reputation);
    };    
    

    /**
	 *  Sets the prices attribute of the allaccept object
	 *
	 *@param  factorID  The new prices value
	 *@param  min       The new prices value
	 *@param  max       The new prices value
	 */
    public synchronized void setPrices(Integer factorID, double min, double max) {
		PriceDistribution d = (PriceDistribution) this.priceHash.get(factorID);
		if (d != null) {
			d.setLowerLimitPrice(Math.min(max, min));
			d.setUpperLimitPrice(Math.max(max, min));
			d.addValue((max + min) / 2);
		} else {
			d = new PriceDistribution((max + min) / 2, 0, Math.min(max, min), Math.max(max, min));
		}
		this.priceHash.put(factorID, d);
    }

    public synchronized double getLowerLimitPrice(Integer factorID) {
		PriceDistribution d = (PriceDistribution) this.priceHash.get(factorID);
		return d.getLowerLimitPrice();
    }

    public void setStrategyOwner(Agent_Source strategyOwner){
		//learning.setLearningOwner(strategyOwner);
		this.strategyOwner=strategyOwner;
    }

	public Agent_Source getStrategyOwner(){
		return this.strategyOwner;
    }

    public synchronized double getUpperLimitPrice(Integer factorID) {
        PriceDistribution d = (PriceDistribution) this.priceHash.get(factorID);
        try {
            return d.getUpperLimitPrice();
        }
        catch(Exception e) { return 0;}
    }

    /**
	 *  Description of the Method
	 *
	 *@param  factorID                  Description of the Parameter
	 *@param  currentPriceDistribution  Description of the Parameter
	 *@param  proposalTypeToGenerate    Description of the Parameter
	 *@return                           Description of the Return Value
	 */
    public synchronized PriceDistribution modifyDealRange(Integer factorID, PriceDistribution currentPriceDistribution, int proposalTypeToGenerate) {
		String METHOD_NAME = "modifyDealRange()";

		if (proposalTypeToGenerate == constant.ASK) {
			//I am buyer, so begin with lower limit than last time, upper limit remains unchanged
			currentPriceDistribution.setInitialPrice(currentPriceDistribution.getWeightedAverage() * (1 - myGenotype.getDeltaJump()), proposalTypeToGenerate);
			//            currentPriceDistribution.setLimitPrice(
			//                    currentPriceDistribution.getWeightedAverage(),
			//                    currentPriceDistribution.getMaximum(),
			//                    proposalTypeToGenerate);

		} else {
			//proposalTypeToGenerate == constant.BID
			//raise initial price, limit price remains unchanged
			currentPriceDistribution.setInitialPrice(currentPriceDistribution.getWeightedAverage() * (1 + myGenotype.getDeltaJump()), proposalTypeToGenerate);
			//            currentPriceDistribution.setLimitPrice(
			//                    currentPriceDistribution.getWeightedAverage(),
			//                    currentPriceDistribution.getMinimum(),
			//                    proposalTypeToGenerate);
		}
		return currentPriceDistribution;
    }

       synchronized PriceDistribution modifyDealRange(Integer factorID,
            PriceDistribution currentPriceDistribution,
            int proposalTypeToGenerate,
            int recommendation) {
        String METHOD_NAME = "modifyDealRange()";

        switch (recommendation) {
            case constant.MSG_Reject:
                switch (proposalTypeToGenerate) {
                    case constant.ASK:
                        currentPriceDistribution.setInitialPrice(
                                Math.max(
                                currentPriceDistribution.getLastAgreementPrice()
                                * (1 + myGenotype.getDeltaJump()),
                                currentPriceDistribution.getUpperLimitPrice()),
                                proposalTypeToGenerate);
                        break;
                    case constant.BID:
                        currentPriceDistribution.setInitialPrice(
                                Math.min(
                                currentPriceDistribution.getLastAgreementPrice()
                                * (1 - myGenotype.getDeltaJump()),
                                currentPriceDistribution.getLowerLimitPrice()),
                                proposalTypeToGenerate);
                        break;
                }

                break;
            case constant.MSG_Accept:
                currentPriceDistribution = modifyDealRange(factorID, currentPriceDistribution, proposalTypeToGenerate);
                break;
            default:
                currentPriceDistribution = modifyDealRange(factorID, currentPriceDistribution, proposalTypeToGenerate);
                break;
        }

        return currentPriceDistribution;
    }

    /**
	 *  Main method of the strategy. Gives back a price to communicate to
	 *  another agent.
         */

    public synchronized double bargain(Integer factorID, int proposalTypeToGenerate, double opponentsCurrentPrice, double myOldPrice, Integer negotiationID) {
    String METHOD_NAME = "bargain()";

        /**
         *  set a fallback return value which will be returned if any of the
         *  following conditions fail
         */
        rec = new Recommendation(constant.MSG_Proposal,
                -1.0,
                "entering bargain()");

        // --------------- these checks are not included in the former allaccept strategy -----

        /**
         *  check whether the factor to be traded is already known to the
         *  strategy. If the good has never been traded before, create a new
         *  entry in the market prices vector.
         */
        PriceDistribution currentPriceDistribution = (PriceDistribution) this.priceHash.get(factorID);
        if (currentPriceDistribution == null) {
            rec.setExecutionState("no market prices known");
            currentPriceDistribution = new PriceDistribution(opponentsCurrentPrice, 0, myOldPrice, myOldPrice);
            currentPriceDistribution.setWeight(myGenotype.getWeightMemory());
            priceHash.put(factorID, currentPriceDistribution);
            //store distribution for later
        }
        
// *********************************************
        if (opponentsCurrentPrice <= 0) {
                rec = new Recommendation(constant.MSG_Proposal, currentPriceDistribution.getWeightedAverage(), "Hola");
                return rec.getPrice();
        } else {
                rec = new Recommendation(constant.MSG_Accept, opponentsCurrentPrice, "Adeu");
                return rec.getPrice();
        }
// *********************************************
    }
	//tey+ 2003-01-01
	/**
	 * Method modifyPrices is used to to update the priceHash table consistently at one place in this class
	 *
	 * @param    factorID            the factor whose prices have to be changed
	 * @param    opponentsCurrentPrice   the new price which has been signaled
	 * @param    proposalTypeToGeneratean whether the price should be raised or lowered
	 * @param    negotiationID       the ordinal number for the negotiation, just for reference and future use
	 *
	 * @version  12/18/2002
	 */
	public void modifyPrices(Integer factorID, double opponentsCurrentPrice, int proposalTypeToGenerate, Integer negotiationID) {
		String METHOD_NAME = "modifyPrices()";
		PriceDistribution currentPriceDistribution = (PriceDistribution) this.priceHash.get(factorID);
		currentPriceDistribution.addValue(opponentsCurrentPrice, myGenotype.getWeightMemory());
		priceHash.put(factorID, this.modifyDealRange(factorID, currentPriceDistribution, proposalTypeToGenerate, rec.getRecommendedAction()));
		this.executionState = "updated streetPrice = " + currentPriceDistribution.getWeightedAverage();
	}
	//tey- 2002-12-18

    /**
	 *  Gets the limitPrice attribute of the IPricing object, which means the
	 *  price UP to which a good is ASKed or DOWN to which a good is BID for
	 *  selling. (The initialPrice is technically the reverse).
	 *
	 *@param  type      Description of the Parameter
	 *@param  factorID  Description of the Parameter
	 *@return           The limitPrice value
	 */

    public synchronized double getLimitPrice(Integer factorID, int type) {
		PriceDistribution p = (PriceDistribution) this.priceHash.get(factorID);

		if (p == null) {
			return -1;
		}

		if (type == constant.BID) {
			return (p.getLowerLimitPrice());
		} else if (type == constant.ASK) {
			return (p.getUpperLimitPrice());
		}
		return 0;
    }


    /**
	 *  Gets the initialPrice attribute of the IPricing object, that means the
	 *  price UP from which a good is BID or DOWN from which a good is ASKed.
	 *  (The limitPrice is technically the reverse).
	 *
	 *@param  factorID  Description of the Parameter
	 *@param  type      Description of the Parameter
	 *@return           The initialPrice value
	 */
    public synchronized double getInitialPrice(Integer factorID, int type) {
		PriceDistribution p = (PriceDistribution) this.priceHash.get(factorID);

		if (p == null) {
			return -1;
		}

		if (type == constant.BID) {
			return (p.getUpperLimitPrice());
		} else if (type == constant.ASK) {
			return (p.getLowerLimitPrice());
		}
		return 0;
    }


    /**
	 *  Description of the Method
	 *
	 *@param  type     Description of the Parameter
	 *@param  myPrice  Description of the Parameter
	 *@param  opPrice  Description of the Parameter
	 *@return          Description of the Return Value
	 */
    protected synchronized boolean crossedOffers(int type, double myPrice, double opPrice) {
		if (type == constant.ASK) {
			return (opPrice <= myPrice);
		} else {
			//type == constant.BID)
			return (opPrice >= myPrice);
		}
    }


    /**
	 *  Gets the averagePrice attribute of the allaccept object
	 *
	 *@param  factorID  Description of the Parameter
	 *@return           The averagePrice value
	 */
    public synchronized double getAveragePrice(Integer factorID) {
		PriceDistribution p = (PriceDistribution) this.priceHash.get(factorID);

		if (p == null) {
			return -1;
		} else {
			return p.getWeightedAverage();
		}
    }


    /**
	 *  Sets the lastAgreementPrice attribute of the allaccept object
	 *
	 *@param  factorID  The new lastAgreementPrice value
	 *@param  price     The new lastAgreementPrice value
	 */
    public synchronized void setLastAgreementPrice(Integer factorID, double price) {
		PriceDistribution p = (PriceDistribution) this.priceHash.get(factorID);
		if (p == null) {
			p = new PriceDistribution(price, 0, price, price);
		}
		p.setLastAgreementPrice(price);
		this.priceHash.put(factorID, p);
    }


    /**
	 *  Gets the negotiationHistory attribute of the allaccept object
	 *
	 *@param  negotiationID  Description of the Parameter
	 *@return                The negotiationHistory value
	 */
    public synchronized NegotiationHistory getNegotiationHistory(String negotiationID) {
		return (NegotiationHistory) this.historyHash.get(negotiationID);
    }


    /**
	 *  Gets the negotiationPartner attribute of the zeroIntelligence object
	 *
	 *@param  groupmembers  Description of the Parameter
	 *@return               The negotiationPartner value
	 */
    public int getNegotiationPartner(ArrayList groupmembers) {
		RandomVariable RND = new RandomVariable(new ContinuousUniformDistribution(0, groupmembers.size()));
		return ((int) RND.simulate());
    }


    /**
	 *  Gets the recommendedAction attribute of the allaccept object
	 *
	 *@return    The recommendedAction value
	 */
    public int getRecommendedAction() {
		//return recommendedAction;
                return rec.getRecommendedAction();
    }


    /**
	 *  Gets the executionState attribute of the allaccept object
	 *
	 *@return    The executionState value
	 */
    public String getExecutionState() {
		//return executionState;
                return rec.getExecutionState();
    }


    /**
	 *  Gets the myGenotype attribute of the allaccept object
	 *
	 *@return    The myGenotype value
	 */
    public Genotype getMyGenotype() {
		return myGenotype;
    }


    /**
	 *  Sets the myGenotype attribute of the allaccept object
	 *
	 *@param  myGenotype  The new myGenotype value
	 */
    public void setMyGenotype(Genotype myGenotype) {
		this.myGenotype = myGenotype;
    }

	/**
	 *  Sets the learningModule attribute of the allaccept object
	 *
	 *@param  learningModule  The new learningModule value
	 */
    public void setLearningModule(ILearning learningModule) {
		this.learning = learningModule;
    }

    public ILearning getLearningModule() {
		return this.learning;
    }

	/**
	 *  Gets the regressionLines attribute of the zeroIntelligence object
	 *
	 *@param  h  Description of the Parameter
	 */
    public void getRegressionLines(java.util.HashMap x, String typ) {};
}
// end of class
