package catnet.strategy;

import catnet.learning.ILearning;
import catnet.accounting.Factor;
import catnet.learning.stdea.Genotype;
import catnet.negotiation.NegotiationHistory;
import catnet.negotiation.PriceDistribution;
import catnet.negotiation.Proposal;
import catnet.data.constant;
import catnet.util.Tool;
import edu.uah.math.distributions.*;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.ArrayList;
import catnet.agent.Agent_Source;
import java.util.Random;
//import com.ls.service.log.Logger;

public class dummy implements IPricing {
    
    private Agent_Source strategyOwner;
    
    /**
     *  Description of the Field
     */
    protected String CLASS_NAME = "dummy";
    
    /**
     *  Spread around the average market price
     */
    protected int ACCEPTABLE_SPREAD = 4;
    
    public ILearning learning = new catnet.learning.stdea.Smith98();
    
    /**
     *  The strategy of the agent contained in a Genotype object.
     *
     *@see    Genotype
     */
    private Genotype myGenotype;
    
    /**
     *  HashMap to hold my own limit and average prices. Key is factorID, Value
     *  is a PriceDistribution object with the price for that good.
     */
    private HashMap priceHash = new HashMap();
    
    /**
     *  Small hashmap to hold the negotiation history. Key is the negotiation
     *  ID, value a NegotationHistory object.
     *
     *@see    NegotationHistory
     */
    private HashMap historyHash = new HashMap();
    
    /**
     *  The type of action this strategy recommends to the pro-active module.
     *
     *@see    IPricing#ACCEPT
     *@see    IPricing#REJECT
     *@see    IPricing#PROPOSE
     */
    private int recommendedAction = 0;
    
    /**
     *  Description of the Field
     */
    private String executionState = "";
    
    /**
     *  cf is the LOCALE currencyFormat to format prices in the logfiles
     */
    private NumberFormat cf;
    
    
    /**
     *  Description of the Field
     */
    private RandomVariable RND = new RandomVariable(new ContinuousUniformDistribution());
    
    
    /**
     *  Constructor for the dummy object
     */
    public dummy() {
        cf = NumberFormat.getNumberInstance();
        //principal = null;
    }
    
    /**
     *  Sets the parameters attribute of the dummy object
     *
     *@param  strategy_blueprint  The new parameters value
     */
    public synchronized void setParameters(HashMap strategy_blueprint) {
        String METHOD_NAME = "setParameters()";
        
        try {
            this.myGenotype = new Genotype(
            new ContinuousUniformDistribution(
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "acquisitiveness/average")) -
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "acquisitiveness/variance")),
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "acquisitiveness/average")) +
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "acquisitiveness/variance"))).simulate(),
            new ContinuousUniformDistribution(
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_step/average")) -
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_step/variance")),
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_step/average")) +
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_step/variance"))).simulate(),
            new ContinuousUniformDistribution(
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_next/average")) -
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_next/variance")),
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_next/average")) +
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "price_next/variance"))).simulate(),
            new ContinuousUniformDistribution(
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "satisfaction/average")) -
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "satisfaction/variance")),
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "satisfaction/average")) +
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "satisfaction/variance"))).simulate(),
            new ContinuousUniformDistribution(
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "weight_memory/average")) -
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "weight_memory/variance")),
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "weight_memory/average")) +
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "weight_memory/variance"))).simulate(),
            new ContinuousUniformDistribution(
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "reputation/average")) -
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "reputation/variance")),
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "reputation/average")) +
            Double.parseDouble((String) Tool.getHashPath(strategy_blueprint, "reputation/variance"))).simulate()
            );
        } catch (NullPointerException ex) {
            this.myGenotype = new Genotype(0.5, 0.5, 0.5, 0.7, 0.5, 1.0);
        }
    }
    
    public void randomizeMyGenotype() {
            Random rnd = new Random();
            //P_acq between 0.4 und 0.8
            //Delta_change between 0.1 and 0.4
            //Delta_jump between 0.1 and 0.6
            //P_sat between 0.4 and 0.8
            //W_mem between 0.3 and 0.7
            //P_rep always 1.0
            
            double p_acquisitiveness=rnd.nextInt(40);
            p_acquisitiveness = 0.4 + p_acquisitiveness /100;
            double del_change=rnd.nextInt(30);
            del_change = 0.1 + del_change/100;
            double del_jump=rnd.nextInt(50);
            del_jump = 0.1 + del_jump/100;
            double p_satisfaction=rnd.nextInt(40);
            p_satisfaction = 0.4 + p_satisfaction/100;
            double w_memory=rnd.nextInt(40);
            w_memory = 0.3 + w_memory/100;
            double p_reputation=1.0;        
            
            this.myGenotype = new Genotype(p_acquisitiveness, del_change, del_jump, p_satisfaction, w_memory, p_reputation);
    };    
    
    
    /**
     *  Sets the prices attribute of the dummy object
     *
     *@param  factorID  The new prices value
     *@param  min       The new prices value
     *@param  max       The new prices value
     */
    public synchronized void setPrices(Integer factorID, double min, double max) {
        PriceDistribution d = (PriceDistribution) this.priceHash.get(factorID);
        if (d != null) {
            d.setLowerLimitPrice(Math.min(max, min));
            d.setUpperLimitPrice(Math.max(max, min));
            d.addValue((max + min) / 2);
        } else {
            d = new PriceDistribution((max + min) / 2, 0, Math.min(max, min), Math.max(max, min));
        }
        this.priceHash.put(factorID, d);
    }
    
    public synchronized double getLowerLimitPrice(Integer factorID) {
        PriceDistribution d = (PriceDistribution) this.priceHash.get(factorID);
        return d.getLowerLimitPrice();
    }
    
    public synchronized double getUpperLimitPrice(Integer factorID) {
        PriceDistribution d = (PriceDistribution) this.priceHash.get(factorID);
        return d.getUpperLimitPrice();
    }
    
    /**
     *  Description of the Method
     *
     *@param  factorID                  Description of the Parameter
     *@param  currentPriceDistribution  Description of the Parameter
     *@param  proposalTypeToGenerate    Description of the Parameter
     *@return                           Description of the Return Value
     */
    public synchronized PriceDistribution modifyDealRange(Integer factorID, PriceDistribution currentPriceDistribution, int proposalTypeToGenerate) {
        String METHOD_NAME = "modifyDealRange()";
        
        if (proposalTypeToGenerate == constant.ASK) {
            //I am buyer, so begin with lower limit than last time, upper limit remains unchanged
            currentPriceDistribution.setInitialPrice(currentPriceDistribution.getLowerLimitPrice(), proposalTypeToGenerate);
        } else {
            //proposalTypeToGenerate == Proposal.BID
            //raise initial price, limit price remains unchanged
            currentPriceDistribution.setInitialPrice(currentPriceDistribution.getLowerLimitPrice(), proposalTypeToGenerate);
        }
        return currentPriceDistribution;
    }
    
    
    /**
     *  Main method of the strategy. Gives back a price to communicate to
     *  another agent.
     *
     *@param  opponentsCurrentPrice   The new price the negotiation opponent has
     *      just communicated.
     *@param  myOldPrice              The last price I have communicated in this
     *      conversation. This parameter is no longer needed as the last price
     *      should be retrieved from historyHash
     *@param  factorID                The factor about which the negotiation
     *      goes.
     *@param  proposalTypeToGenerate  If I want to buy = Proposal.ASK, If I want
     *      to sell = Proposal.BID
     *@param  negotiationID           The unique identification of the ongoing
     *      negotiation
     *@return                         The price to be communicated to the
     *      negotiation opponent
     */
    
    public synchronized double bargain(Integer factorID, int proposalTypeToGenerate, double opponentsCurrentPrice, double myOldPrice, Integer negotiationID) {
        String METHOD_NAME = "bargain";
        double myNewPrice = -1.0;
        this.executionState = "entering Bargain";
        PriceDistribution currentPriceDistribution = (PriceDistribution) this.priceHash.get(factorID);
        
        /**
         *  this is the first time we talk about that particular factor. Two
         *  possibilities exist here, either to break the negotiation or to
         *  create a new price impression on that factor. We choose the second
         *  possibility.
         *
         *@todo    code workaround for prices = -1
         */
        if (currentPriceDistribution == null) {
            this.executionState = "no market prices known";
            currentPriceDistribution = new PriceDistribution(opponentsCurrentPrice, 0, myOldPrice, myOldPrice);
            currentPriceDistribution.setWeight(myGenotype.getWeightMemory());
            priceHash.put(factorID, currentPriceDistribution);
        }
        // after this line, a valid price distribution exists for that factor
        
        /**
         *  Now check whether we know about that particular negotiation. If
         *  there is no such negotiation, generate a new dataset for use later
         *  and return from bargain() with an initial offer. Two different
         *  possibilities exist here. Either the opponent has given us his idea
         *  about a valid price in opponentsCurrentPrice. If he has no idea,
         *  then opponentsCurrentPrice = -1.0.
         */
        NegotiationHistory currentNegotiationHistory = (NegotiationHistory) historyHash.get(negotiationID);
        if (currentNegotiationHistory == null) {
            this.executionState = "generate new negotiation";
            this.recommendedAction = constant.MSG_Proposal;
            myNewPrice = this.getInitialPrice(factorID, proposalTypeToGenerate);
            //System.out.println("initialprice: "+myNewPrice);
            if (opponentsCurrentPrice <= 0) {
                //proposal is cfp with no price given, so do not calculate price step yet
                historyHash.put(negotiationID, new NegotiationHistory(
                myNewPrice,
                opponentsCurrentPrice,
                myNewPrice,
                opponentsCurrentPrice,
                -1.0
                ));
            } else {
                //valid counterproposal, calculate everything
                double step = Math.abs(opponentsCurrentPrice - myNewPrice) * myGenotype.getDeltaChange();
                historyHash.put(negotiationID,
                new NegotiationHistory(myNewPrice, opponentsCurrentPrice, myNewPrice, opponentsCurrentPrice, step));
                
                currentNegotiationHistory = (NegotiationHistory) historyHash.get(negotiationID);
                
                //check if limit price already crosses oponents offer
                //if so, then accept the opponents offer immediately
                if (this.crossedOffers(proposalTypeToGenerate, myNewPrice, opponentsCurrentPrice) &&
                opponentsCurrentPrice > 0) {
                    myNewPrice = opponentsCurrentPrice;
                    recommendedAction = constant.MSG_Accept;
                    currentPriceDistribution.setLastAgreementPrice(myNewPrice);
                    
                    currentPriceDistribution.addValue(opponentsCurrentPrice, myGenotype.getWeightMemory());
                    //                    priceHash.put(factorID, this.modifyDealRange(factorID, currentPriceDistribution, proposalTypeToGenerate));
                    priceHash.put(factorID, currentPriceDistribution);
                    this.executionState = "updated streetPrice = " + currentPriceDistribution.getWeightedAverage();
                    
                    priceHash.put(factorID, currentPriceDistribution);
                    
                    currentNegotiationHistory.setMyLastPrice(myNewPrice);
                    currentNegotiationHistory.setOpponentsLastPrice(opponentsCurrentPrice);
                    historyHash.put(negotiationID, currentNegotiationHistory);
                    this.executionState = "offers have been crossed";
                }
            }
            return myNewPrice;
        }
        /**
         *  After this line, we should have a valid counterproposal of the
         *  opponent. If opponentsCurrentPrice = -1, then reject immediately
         */
        if (opponentsCurrentPrice <= 0) {
            recommendedAction = constant.MSG_Reject;
            //System.out.println("CurrentPrice < 0");
            executionState = "opponent sent no counterproposal";
            return opponentsCurrentPrice;
        }
        /**
         *  One thing for housekeeping is left: the opponentsLastPrice may have
         *  been -1 (a cfp with no price information given). As we are going now
         *  into the second round, some variables have to be replaced.
         */
        if (currentNegotiationHistory.getOpponentsLastPrice() == -1) {
            currentNegotiationHistory.setOpponentsStartPrice(opponentsCurrentPrice);
            double step = Math.abs(opponentsCurrentPrice - currentNegotiationHistory.getMyStartPrice()) * myGenotype.getDeltaChange();
            currentNegotiationHistory.setPriceStep(step);
        }
        /**
         *  After this line we know about the particular negotiation and all
         *  variables have been filled. The next thing to check is whether the
         *  counteroffer has crossed our offer, and we can immediately accept
         *  without having to negotiate further.
         */
        myOldPrice = currentNegotiationHistory.getMyLastPrice();
        if (this.crossedOffers(proposalTypeToGenerate, myOldPrice, opponentsCurrentPrice)) {
            
            myNewPrice = opponentsCurrentPrice;
            recommendedAction = constant.MSG_Accept;
            currentPriceDistribution.setLastAgreementPrice(myNewPrice);
            currentPriceDistribution.addValue(opponentsCurrentPrice, myGenotype.getWeightMemory());
            //            priceHash.put(factorID, this.modifyDealRange(factorID, currentPriceDistribution, proposalTypeToGenerate));
            priceHash.put(factorID, currentPriceDistribution);
            this.executionState = "updated streetPrice = " + currentPriceDistribution.getWeightedAverage();
            System.out.println(" #" + negotiationID
            + ". street price has changed. "
            + "(w_avg/avg/lower/upper/last_agr): ("
            + cf.format(currentPriceDistribution.getWeightedAverage()) + "/"
            + cf.format(currentPriceDistribution.getMeanAverage()) + "/"
            + cf.format(currentPriceDistribution.getLowerLimitPrice()) + "/"
            + cf.format(currentPriceDistribution.getUpperLimitPrice()) + "/"
            + cf.format(currentPriceDistribution.getLastAgreementPrice()) + ")"
            );
            priceHash.put(factorID, currentPriceDistribution);
            currentNegotiationHistory.setMyLastPrice(myNewPrice);
            currentNegotiationHistory.setOpponentsLastPrice(opponentsCurrentPrice);
            historyHash.put(negotiationID, currentNegotiationHistory);
            this.executionState = "offers have been crossed";
            if (proposalTypeToGenerate == constant.ASK) {
                recommendedAction = constant.MSG_Proposal;
            }
            return myNewPrice;
        }
        
        // first strategy component: get proposal value and update street price accordingly, if opponent has communicated a valid price quote (>0)
        /**
         *@todo    only last price after rejectance or accept, not ongoing
         *      negotiation prices if (opponentsCurrentPrice > 0) {
         *      currentPriceDistribution.addValue(opponentsCurrentPrice,
         *      myGenotype.getWeightMemory()); this.executionState = "updated
         *      streetPrice = " + currentPriceDistribution.getWeightedAverage();
         *      principal.printOut(CLASS_NAME, Logger.INFO, METHOD_NAME, " #" +
         *      negotiationID + ". street price has changed. " +
         *      "(w_avg/avg/lower/upper/last_agr): (" +
         *      cf.format(currentPriceDistribution.getWeightedAverage()) + "/" +
         *      cf.format(currentPriceDistribution.getMeanAverage()) + "/" +
         *      cf.format(currentPriceDistribution.getLowerLimitPrice()) + "/" +
         *      cf.format(currentPriceDistribution.getUpperLimitPrice()) + "/" +
         *      cf.format(currentPriceDistribution.getLastAgreementPrice()) +
         *      ")" ); priceHash.put(factorID, currentPriceDistribution); }
         */
        // second strategy component: check whether we are satisfied with the offer or downright refuse it
        double price_deviation = Math.abs(opponentsCurrentPrice - currentPriceDistribution.getWeightedAverage());
        /**
         *@todo    build acceptance boundaries and check if offer is between
         */
        double upperBound = currentPriceDistribution.getUpperLimitPrice();
        double lowerBound = currentPriceDistribution.getLowerLimitPrice();
        if /*(*/(opponentsCurrentPrice >= upperBound)/* || (opponentsCurrentPrice <= lowerBound))*/ {
            // unconditional constraint: usury; opponent's price is 100% off the street price
            recommendedAction = constant.MSG_Reject;
            //System.out.println("upperbound: "+upperBound+" lowerbound: "+lowerBound+" opponentsCurrentPrice: "+opponentsCurrentPrice);
            this.executionState = "rejected because of usury";
            myNewPrice = opponentsCurrentPrice;
            currentNegotiationHistory.setMyLastPrice(myNewPrice);
            currentNegotiationHistory.setOpponentsLastPrice(opponentsCurrentPrice);
            historyHash.put(negotiationID, currentNegotiationHistory);
            priceHash.put(factorID, currentPriceDistribution);
            return myNewPrice;
        }
        
        // fourth strategy component: if we make a concession, calculate the step
        recommendedAction = constant.MSG_Proposal;
        double delta_price = currentPriceDistribution.getWeightedAverage()/10;
        if (proposalTypeToGenerate == constant.ASK) {
            // I am buyer, opponent is seller
            this.executionState = "price raised by concession of " + cf.format(delta_price);
            //do not ask more than the upper bound set
            // Pau changes on negotiation
            if (delta_price < 0)
                delta_price = - delta_price;
            myNewPrice = Math.min(currentPriceDistribution.getUpperLimitPrice(), myOldPrice + delta_price);
            //do not ask more than the opponent has bid
            myNewPrice = Math.min(opponentsCurrentPrice, myNewPrice);
            currentNegotiationHistory.setMyLastPrice(myNewPrice);
            currentNegotiationHistory.setOpponentsLastPrice(opponentsCurrentPrice);
            historyHash.put(negotiationID, currentNegotiationHistory);
            priceHash.put(factorID, currentPriceDistribution);
            return myNewPrice;
        }
        if (proposalTypeToGenerate == constant.BID) {
            // I am seller, opponent is buyer
            this.executionState = "price lowered by concession of " + cf.format(delta_price);
            //do not bid less than the lower limit price
            // Pau changes on negotiation
            if (delta_price < 0)
                delta_price = - delta_price;
            myNewPrice = Math.max(currentPriceDistribution.getLowerLimitPrice(), myOldPrice - delta_price);
            //do not bid less than opponent has asked
            myNewPrice = Math.max(opponentsCurrentPrice, myNewPrice);
            currentNegotiationHistory.setMyLastPrice(myNewPrice);
            currentNegotiationHistory.setOpponentsLastPrice(opponentsCurrentPrice);
            historyHash.put(negotiationID, currentNegotiationHistory);
            priceHash.put(factorID, currentPriceDistribution);
            return myNewPrice;
        }
        // end of seller_delta
        
        // end of p_acq passed (else branch)
        //termination of strategy: if nothing else holds, repeat last price
        this.executionState = "no condition applied, repeat last price";
        myNewPrice = currentNegotiationHistory.getMyLastPrice();
        currentNegotiationHistory.setMyLastPrice(myNewPrice);
        currentNegotiationHistory.setOpponentsLastPrice(opponentsCurrentPrice);
        historyHash.put(negotiationID, currentNegotiationHistory);
        priceHash.put(factorID, currentPriceDistribution);
        return myNewPrice;
    }
    // end of bargain method
    /**
     *  Gets the limitPrice attribute of the IPricing object, which means the
     *  price UP to which a good is ASKed or DOWN to which a good is BID for
     *  selling. (The initialPrice is technically the reverse).
     *
     *@param  type      Description of the Parameter
     *@param  factorID  Description of the Parameter
     *@return           The limitPrice value
     */
    
    public synchronized double getLimitPrice(Integer factorID, int type) {
        PriceDistribution p = (PriceDistribution) this.priceHash.get(factorID);
        
        if (p == null) {
            return -1;
        }
        
        if (type == constant.BID) {
            return (p.getLowerLimitPrice());
        } else if (type == constant.ASK) {
            return (p.getUpperLimitPrice());
        }
        return 0;
    }
    
    
    /**
     *  Gets the initialPrice attribute of the IPricing object, that means the
     *  price UP from which a good is BID or DOWN from which a good is ASKed.
     *  (The limitPrice is technically the reverse).
     *
     *@param  factorID  Description of the Parameter
     *@param  type      Description of the Parameter
     *@return           The initialPrice value
     */
    public synchronized double getInitialPrice(Integer factorID, int type) {
        PriceDistribution p = (PriceDistribution) this.priceHash.get(factorID);
        
        if (p == null) {
            return -1;
        }
        
        if (type == constant.BID) {
            return (p.getUpperLimitPrice());
        } else if (type == constant.ASK) {
            return (p.getLowerLimitPrice());
        }
        return 0;
    }
    
    
    /**
     *  Description of the Method
     *
     *@param  type     Description of the Parameter
     *@param  myPrice  Description of the Parameter
     *@param  opPrice  Description of the Parameter
     *@return          Description of the Return Value
     */
    protected synchronized boolean crossedOffers(int type, double myPrice, double opPrice) {
        if (type == constant.ASK) {
            return (opPrice <= myPrice);
        } else {
            //type == constant.BID)
            return (opPrice >= myPrice);
        }
    }
    
    
    /**
     *  Gets the averagePrice attribute of the dummy object
     *
     *@param  factorID  Description of the Parameter
     *@return           The averagePrice value
     */
    public synchronized double getAveragePrice(Integer factorID) {
        PriceDistribution p = (PriceDistribution) this.priceHash.get(factorID);
        
        if (p == null) {
            return -1;
        } else {
            return p.getWeightedAverage();
        }
    }
    
    
    /**
     *  Sets the lastAgreementPrice attribute of the dummy object
     *
     *@param  factorID  The new lastAgreementPrice value
     *@param  price     The new lastAgreementPrice value
     */
    public synchronized void setLastAgreementPrice(Integer factorID, double price) {
        PriceDistribution p = (PriceDistribution) this.priceHash.get(factorID);
        if (p == null) {
            p = new PriceDistribution(price, 0, price, price);
        }
        p.setLastAgreementPrice(price);
        this.priceHash.put(factorID, p);
    }
    
    
    /**
     *  Gets the negotiationHistory attribute of the dummy object
     *
     *@param  negotiationID  Description of the Parameter
     *@return                The negotiationHistory value
     */
    public synchronized NegotiationHistory getNegotiationHistory(String negotiationID) {
        return (NegotiationHistory) this.historyHash.get(negotiationID);
    }
    
    
    /**
     *  Gets the negotiationPartner attribute of the zeroIntelligence object
     *
     *@param  groupmembers  Description of the Parameter
     *@return               The negotiationPartner value
     */
    public int getNegotiationPartner(ArrayList groupmembers) {
        RandomVariable RND = new RandomVariable(new ContinuousUniformDistribution(0, groupmembers.size()));
        return ((int) RND.simulate());
    }
    
    
    /**
     *  Gets the recommendedAction attribute of the dummy object
     *
     *@return    The recommendedAction value
     */
    public int getRecommendedAction() {
        return recommendedAction;
    }
    
    
    /**
     *  Gets the executionState attribute of the dummy object
     *
     *@return    The executionState value
     */
    public String getExecutionState() {
        return executionState;
    }
    
    
    /**
     *  Gets the myGenotype attribute of the dummy object
     *
     *@return    The myGenotype value
     */
    public Genotype getMyGenotype() {
        return myGenotype;
    }
    
    
    /**
     *  Sets the myGenotype attribute of the dummy object
     *
     *@param  myGenotype  The new myGenotype value
     */
    public void setMyGenotype(Genotype myGenotype) {
        this.myGenotype = myGenotype;
    }
    
    /**
     *  Sets the learningModule attribute of the dummy object
     *
     *@param  learningModule  The new learningModule value
     */
    public void setLearningModule(ILearning learningModule) {
        this.learning = learningModule;
    }
    
    
    /**
     *  Gets the learningModule attribute of the BAgent object
     *
     *@return    The learningModule value
     */
    public ILearning getLearningModule() {
        return this.learning;
    }
    
    /**
     *  Gets the regressionLines attribute of the zeroIntelligence object
     *
     *@param  h  Description of the Parameter
     */
    public void getRegressionLines(java.util.HashMap x, String typ) {};
    
    public void setStrategyOwner(Agent_Source strategyOwner){
        //learning.setLearningOwner(strategyOwner);
        this.strategyOwner=strategyOwner;
    }
    
    public Agent_Source getStrategyOwner(){
        return this.strategyOwner;
    }
    
    
}
// end of class
