package catnet.util;

import java.util.*;
import drcl.inet.InetPacket;
import drcl.inet.Node;
import drcl.comp.Port;
import drcl.comp.Component;
import drcl.comp.contract.GarbageContract;
import drcl.inet.contract.DatagramContract;
import catnet.data.pkt;


/**
 * CatNet Monitor
 *
 * @author Roc Messeguer
 * @version 0.2, 18/12/2002
 */

public class CatNetMonitor extends drcl.comp.Extension
{
	
	Port out = addPort("output");
	Port in = addPort("in");
	public static long packets = 0;
	public static long hops = 0;
	boolean accounting = true;
	
	
	public CatNetMonitor() 
	{ super(); }
	
	
	public CatNetMonitor(String id_)
	{ super(id_); }
	
	
	public void reset()
	{
		super.reset();
		packets = 0;
                hops = 0;
	}
	
	
	public void duplicate(Object source_)
	{
		super.duplicate(source_);
	}
	
	protected static void increasePackets() {
            packets++;
        }
        
	protected static void increaseHops(long howMany) {
            hops += howMany;
        }
        
	protected synchronized void process(Object data_, Port inPort_) 
	{

		if (data_ instanceof GarbageContract.Message)
			data_ = ((GarbageContract.Message)data_).getData();

		if (data_ instanceof InetPacket) {
			// Obtain Packet from incoming data
			InetPacket p_ = (InetPacket)data_;

			if (accounting) {
                            increasePackets();
                            increaseHops(p_.getHops() + 1);
			}
/*                        
			out.doSending( getTime()
//					+ " s " + p_.getSource()
//					+ " d " + p_.getDestination()
					+ "\t" + p_.getHops()
                                        + "\t" + ((catnet.data.pkt) ((drcl.inet.transport.UDPPacket) p_.getBody()).getBody()).type
					//+ " p " + p_.getPacketType()
					+ "\t" + p_.getPacketSize()
//					+ " i " + p_.getID()
//+ "\t" + ((catnet.data.pkt) ((drcl.inet.transport.UDPPacket) p_.getBody()).getBody()).msg.getConversation_ID()
		 			+ "\r\n");			
 */
		}
/*
		if (data_ instanceof DatagramContract.Message)
			data_ = ((DatagramContract.Message)data_).getContent();
		
                if (data_ instanceof pkt)
		{
			// Obtain CatNet Packet from incoming data
			pkt pkt_ = (pkt) data_;

			out.doSending( " c " + pkt_.type
					+ "\n");			
		}
 */
	}


	/** 
	 * Sets true to enable accounting. 
	 */
	 
	public void setAccountingEnabled(boolean enabled_)
	{ accounting = enabled_; }


	/** 
	 * Returns true if accounting is enabled. 
	 */
	 
	public boolean isAccountingEnabled()
	{ return accounting; }
	
		
	public String info()
	{
		if (accounting)
			return "   # packets = " + packets + "\n";
		else
			return "Accounting is disabled.";
	}

	
	/**
	 * Sets up a CatNetMonitor on a network component.
	 * 
	 * @param net_ 
	 * @param filename_
	 */	 
	 
	public static CatNetMonitor setCatNetMonitorOn(Component net_, String filename_)
	{
		CatNetMonitor catnet_ = new CatNetMonitor("._catnet_");
//		drcl.comp.io.FileComponent file_ = new drcl.comp.io.FileComponent("._file_");
//		file_.open(filename_);
		net_.addComponent(catnet_);
//		net_.addComponent(file_);
//		catnet_.getPort("output").connectTo(file_.addPort("in"));
		setCatNetMonitorOn(net_, catnet_);
		return catnet_;
	}


	/**
	 * Sets up a CatNetMonitor on a network component.
	 * 
	 * @param net_ 
	 * @param monitor_
	 */

	public static void setCatNetMonitorOn(Component net_, CatNetMonitor monitor_)
	{
		try {
		Component[] comp_ = net_.getAllComponents();
		Hashtable ht_ = null;
		int i;

		for (i=0; i<comp_.length; i++) {
			try {
				if (comp_[i] instanceof Node) {
					Node n_ = (Node)comp_[i];
					Port[] ppp_ = n_.getCSL().getAllPorts("up");
					for (int j=0; j<ppp_.length; j++) {
						ppp_[j].attachOut(monitor_.getPort("in"));
					}
				}
			}
			catch (Exception e_) {
				// ignore this component, just continue
			}
		}
		}
		catch (Exception e_) {
			e_.printStackTrace();
		}
	}
}
