package catnet.util;

import java.util.*;
import drcl.comp.Component;
import drcl.inet.Node;
import catnet.servicecopy.ServiceCopy;
import catnet.client.Client;
import catnet.resource.Resource;
import catnet.data.constant;

/**
 * Dynamic Topology
 *
 * @author Roc Messeguer
 * @version 0.1, 18/12/2002
 */

public class DynamicTopology extends drcl.net.Module //drcl.comp.Extension
{
    private Vector vectorSC_;
    private Vector vectorC_;
    private Vector vectorR_;
    private int lastDemmands = -1;
    private int sameDemmands = 0;
    private static boolean pendingdemmands = true;
    
    public DynamicTopology() {
        super();
        vectorSC_ = new Vector();
        vectorC_ = new Vector();
        vectorR_ = new Vector();
    }
    
    
    public DynamicTopology(String id_) {
        super(id_);
        vectorSC_ = new Vector();
        vectorC_ = new Vector();
        vectorR_ = new Vector();
    }
        
    public void initTimer() {
        double now_ = getTime();
        double time_ = now_ + new Float(constant.changeTime).floatValue()/1000;
        TM_EVT dynamicChangeEvent = null;
        dynamicChangeEvent = new TM_EVT(constant.DYNAMIC_CHANGE, 0, time_);
        setTimeoutAt(dynamicChangeEvent, time_);
    }
    
    public void reset() {
        super.reset();
    }
    
    
    public String info() {
        return vectorSC_.toString();
    }
    
    
    /**
     * Sets index SC to enable.
     */
    
    public void setSCEnabled(int index_) {
        System.out.println(vectorSC_.get(index_));
        ((ServiceCopy) vectorSC_.get(index_)).enableServiceCopy(true);
    }
    
    
    /**
     * Sets index SC to disable.
     */
    
    public void setSCDisabled(int index_) {
        System.out.println(vectorSC_.get(index_));
        ((ServiceCopy) vectorSC_.get(index_)).enableServiceCopy(false);
    }
    
    /**
     * Sets up DynamicTopology on a network component.
     *
     * @param net_
     */

    public void setDynamicTopologyOn(Component net_) {
        try {
            Component[] comp_ = net_.getAllComponents();
            int i;
            
            for (i=0; i<comp_.length; i++) {
                try {
                    if (comp_[i] instanceof Node) {
                        Component[] comp2_ = comp_[i].getAllComponents();
                        int j;
                        for (j=0; j<comp2_.length; j++) {
                            try {
                                if (comp2_[j] instanceof ServiceCopy) {
                                    if (((ServiceCopy) comp2_[j]).enabled)
                                        vectorSC_.addElement(comp2_[j]);
                                }
                                else if (comp2_[j] instanceof Client) {
                                    vectorC_.addElement(comp2_[j]);
                                }
                                else if (comp2_[j] instanceof Resource) {
                                    vectorR_.addElement(comp2_[j]);
                                }
                            }
                            catch (Exception e_) {
                            }
                        }
                    }
                }
                catch (Exception e_) {
                    // ignore this component, just continue
                }
            }
        }
        catch (Exception e_) {
            e_.printStackTrace();
        }
        // Initalizing dynamic change timer
//        initTimer();
        
            TM_EVT testEvent = null;
            testEvent = new TM_EVT(constant.DYNAMIC_CHANGE, 0, getTime());
            timeout(testEvent);
            
        // Initalizing status update timer
        if ((constant.m_baseline) && (constant.mscupdates)) {
            double now_ = getTime();
            double time_ = now_ + new Float(constant.MSCUpdateTime).floatValue();
            TM_EVT statusUpdateEvent = null;
            statusUpdateEvent = new TM_EVT(constant.UPDATE_STATUS, 0, time_);
            setTimeoutAt(statusUpdateEvent, time_);
        }
    }
    
    
    protected synchronized void timeout(Object evt_) {
        int resourceId = ((TM_EVT)evt_).resourceid;
        double timeout = ((TM_EVT)evt_).timeout;
        //debug("agent "+resourceId+": timeout  "+timeout);
        
        // Check if timeout was being reset or cancelled
        if (Double.isNaN(timeout))
            return;
        // XX: rounding error if in real-time simulation?
        if (timeout > getTime()) {
            setTimeoutAt(evt_, timeout);
            return;
        }
        
        switch(((TM_EVT)evt_).type) {
            case constant.DYNAMIC_CHANGE: {

                // 2003-03-06 - New version of dynamism
                // Now instead of changing the state of % we are gonna change the same SC to on and to off 
                // to mantain always the same number of nodes on. The thing that going to change is this
                // number
                
//                Random rnd = new Random(new Double(getTime()*1000).intValue());
                java.util.Date now = new java.util.Date();
                Random rnd = new Random(now.getTime());
                int nChangedOn = 0;
                int nChangedOff = 0;
                int nChangeOn, nChangeOff;
                
                int currOn = 0, currOff = 0;
                {
                    for (int i=0; i<vectorSC_.size(); i++) {
                        if (((ServiceCopy) vectorSC_.get(i)).isEnabledServiceCopy()) {
                            currOn++;
                        } else {
                            currOff++;
                            // Si est sirviendo lengthdecimos que luego lo recupere
                            ((ServiceCopy) vectorSC_.get(i)).addStopTime();                            
                        }
                    }
//                    debug("dyn - Before change SCOn = " + currOn + " (" + (currOn*100)/vectorSC_.size() + ") & SCOff = " + currOff + " (" + (currOff*100)/vectorSC_.size() + ")");
                }
 
                String resourceBW = new String();
                int j = 0;
                while (j<vectorR_.size()) {
//debug("mem - Resource " + j + " has " + ((Resource) vectorR_.get(j)).negotiations.size() + " pending negotiations -> " + ((Resource) vectorR_.get(j)).negotiations.toString());
//debug("mem - Resource " + j + " has " + ((Resource) vectorR_.get(j)).forwarded_Conversations.size() + " forwarded_Conversations");                    
                    if (((Resource) vectorR_.get(j)).knownServiceCopies.size() > 0)
                        resourceBW += "\t" + (((Resource) vectorR_.get(j)).SCBWUnits + ((Resource) vectorR_.get(j)).currentRecovering.size()*2);
//                        resourceBW += "\t" + ((Resource) vectorR_.get(j)).SCBWUnits + "(" + ((Resource) vectorR_.get(j)).LOCALHP.h + ")";
                    j++;
                }
//                debug("rbw - " + resourceBW);
                System.out.println("rbw - " + resourceBW);
                
                if (lastDemmands == -1) {
                    nChangeOn = 0;
                    nChangeOff = (vectorSC_.size()*constant.initialPercentOff)/100;
                    int rest = (vectorSC_.size()*constant.initialPercentOff)%100;
                    if (rest >= 50)
                        nChangeOff++;
                } else {
                    nChangeOn = (vectorSC_.size()*constant.changeProbability)/200;
                    int restProb = rnd.nextInt(199);
                    int rest = (vectorSC_.size()*constant.changeProbability)%200;
                    if (rest > restProb)
                        nChangeOn++;
                    nChangeOn = java.lang.Math.min(nChangeOn, java.lang.Math.min(currOn, currOff));
                    nChangeOff = nChangeOn;
                }
//debug("status - currently " + currOn + " SC are ON and " + (nChangeOn + nChangeOff) + " are going to change");
//                    debug("dyn - Dynamism - Setting " + nChangeOn + " SC to On and " + nChangeOff + " to Off from " + vectorSC_.size() + " SCs");

                Set changedSet = new HashSet();
//debug("dyn - On: " + nChangedOn + "/" + nChangeOn + " Off: " + nChangedOff + "/" + nChangeOff);
                while((nChangedOn < nChangeOn) || (nChangedOff < nChangeOff)) {
                    int currNode = rnd.nextInt(vectorSC_.size());
                    boolean currEnabled = ((ServiceCopy) vectorSC_.get(currNode)).isEnabledServiceCopy();
                    // To avoid some "strange" errors
                    if (changedSet.contains(new Integer(currNode)))
                        continue;
                    
                    // If it's enabled and we should change more SC to Off
                    if (currEnabled && (nChangedOff < nChangeOff)) {
                        ServiceCopy tmp = (ServiceCopy) vectorSC_.get(currNode);
                         tmp.enableServiceCopy(false);
                            nChangedOff++;
                            changedSet.add(new Integer(currNode));
//debug("dyn - To Off " + tmp.LOCALHP.h);
                    }
                    // If it's not enabled and we should change more SC to On
                    if ( !currEnabled && (nChangedOn < nChangeOn)) {
                        ((ServiceCopy) vectorSC_.get(currNode)).enableServiceCopy(true);
                        nChangedOn++;
                            changedSet.add(new Integer(currNode));
//debug("dyn - To On " + currNode);
                    }
                }

                {
                    currOn = 0;
                    currOff = 0;
                    for (int i=0; i<vectorSC_.size(); i++) {
                        if (((ServiceCopy) vectorSC_.get(i)).isEnabledServiceCopy()) {
                            currOn++;
//debug("status - rbw - SC " + ((ServiceCopy) vectorSC_.get(i)).LOCALHP.h + " SC is ON");
                        } else {
                            currOff++;
//debug("status - rbw - SC " + ((ServiceCopy) vectorSC_.get(i)).LOCALHP.h + " SC is OFF");
                        }
                    }
//                    debug("dyn - After change SCOn = " + currOn + " (" + (currOn*100)/vectorSC_.size() + ") & SCOff = " + currOff + " (" + (currOff*100)/vectorSC_.size() + ")");
//                    debug("dyn - end");
                }
                
                j = 0;
                int currDemmands = 0;
                while (j<vectorC_.size()) {
                    currDemmands += ((Client) vectorC_.get(j)).pendingDemands.size();
                    j++;
                }

                // Miramos si el anterior timeout teniamos las mismas demandas y si es asi
                // incrementamos el contador
                if (currDemmands == lastDemmands)
                    sameDemmands ++;
                else
                    sameDemmands = 0;
                lastDemmands = currDemmands;
                
                if (currDemmands == 0)
                    pendingdemmands = false;
                
                // Si el contador no ha cambiado en 10 timeouts paramos
                if ((sameDemmands < 10000) && (pendingdemmands)) {
                    initTimer();
                }
                else {
                    constant.stopsimulation = true;
//debug("rbw - " + ((Resource) vectorR_.get(1)).recoveredStop);
                    debug("Ending simulation - SWF = \t" + constant.SWF);
                    catnet.util.CatNetMonitor catMon = (catnet.util.CatNetMonitor) (this.getParent()).getComponent("._catnet_");
                    if (catMon != null) {
                        debug("CC:\t" + catMon.packets + "\t" + catMon.hops);
                    } else {
                        debug("CC:\t-1\t-1");
                    }
                }
                break;
            }
            case constant.UPDATE_STATUS: {
                for(int i = 0; i<vectorSC_.size(); i++)
                    ((ServiceCopy) vectorSC_.get(i)).sendStatus();
                
                if (pendingdemmands) {
                    double now_ = getTime();
                    double time_ = now_ + new Float(constant.MSCUpdateTime).floatValue();
                    TM_EVT statusUpdateEvent = null;
                    statusUpdateEvent = new TM_EVT(constant.UPDATE_STATUS, 0, time_);
                    setTimeoutAt(statusUpdateEvent, time_);
                }
            }
        }
    }
    
}
