package catnet.util;

import java.util.ArrayList;
import java.util.Arrays;

/**
 *  berschrift: B2B-OS Beschreibung: Copyright: Copyright (c) 2001
 *  Organisation: IIG Telematik, Albert-Ludwigs-Universitaet Freiburg
 *
 *@author     Dr. Torsten Eymann
 *@created    23. Januar 2002
 *@version    1.0
 */

public class RingArray extends ArrayList implements Cloneable {

    /**
     *  Description of the Field
     */
    private int pointer;
    /**
     *  Description of the Field
     */
    private double minimum;
    /**
     *  Description of the Field
     */
    private double maximum;
    /**
     *  Description of the Field
     */
    private double mean, pvariance;

    /**
     *  Description of the Field
     */
    private static int DEFAULT_SIZE = 30;

    /**
     *  Description of the Field
     */
    private int capacity;


    /**
     *  Constructor for the RingArray object
     */
    public RingArray() {
        capacity = DEFAULT_SIZE;
        pointer = 0;
    }


    /**
     *  Constructor for the RingArray object
     *
     *@param  size  Description of the Parameter
     */
    public RingArray(int size) {
        capacity = size;
        pointer = 0;
    }


    /**
     *  Adds a feature to the Value attribute of the RingArray object
     *
     *@param  v  The feature to be added to the Value attribute
     */
    public synchronized void addValue(double v) {

        if (this.isEmpty()) {
            mean = v;
            maximum = v;
            minimum = v;

        } else if (this.size() < capacity) {
            //array is not yet full
            this.add(new Double(v));
            calculateStatistics();
            pointer = this.size();

        } else {
            //array is full, remove least recent data first before adding
            incrementPointer();
            double leastRecent = ((Double) this.get(pointer)).doubleValue();
            this.add(pointer, (new Double(v)));
            calculateStatistics();
        }
    }


    /**
     *  Description of the Method
     */
    private void incrementPointer() {
        pointer = (pointer + 1) % capacity;
    }



    /**
     *  Description of the Method
     */
    public synchronized void calculateStatistics() {
        double sum = 0;
        double v = -1;
        double m = 0;
        double msqr = 0;
        for (int i = 0; i < this.size(); i++) {
            v = ((Double) this.get(i)).doubleValue();
            sum += v;
            maximum = Math.max(this.maximum, v);
            minimum = Math.min(this.minimum, v);
            m = ((double) (i - 1) / i) * m + v / i;
            msqr = ((double) (i - 1) / i) * msqr + v * v / i;
        }
        mean = m;
        pvariance = Math.max(0, msqr - mean * mean);
    }


    /**
     *  Gets the minimum attribute of the RingArray object
     *
     *@return    The minimum value
     */
    public double getMinimum() {
        return minimum;
    }


    /**
     *  Gets the maximum attribute of the RingArray object
     *
     *@return    The maximum value
     */
    public double getMaximum() {
        return maximum;
    }


    /**
     *  Gets the average attribute of the RingArray object
     *
     *@return    The average value
     */
    public double getAverage() {
        return mean;
    }


    /**
     *  Gets the mean attribute of the RingArray object
     *
     *@return    The mean value
     */
    public double getMean() {
        return mean;
    }


    /**
     *  Get the population standard deviation
     *
     *@return    The pSD value
     */
    public double getPSD() {
        return Math.sqrt(this.pvariance);
    }


    /**
     *  Gets the variance attribute of the RingArray object
     *
     *@return    The variance value
     */
    public double getVariance() {
        return ((double) size() / (size() - 1)) * this.pvariance;
    }


    /**
     *  Gets the sD attribute of the RingArray object
     *
     *@return    The sD value
     */
    public double getSD() {
        return Math.sqrt(getVariance());
    }



    /**
     *  Gets the pointer attribute of the RingArray object
     *
     *@return    The pointer value
     */
    public int getPointer() {
        return pointer;
    }


    /**
     *  Sets the pointer attribute of the RingArray object
     *
     *@param  pointer  The new pointer value
     */
    public void setPointer(int pointer) {
        this.pointer = pointer;
    }

}
